package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.apache.commons.lang.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdHospital;
import org.jeecg.modules.basedata.entity.FdHospitalContacts;
import org.jeecg.modules.basedata.service.IFdHospitalContactsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 医院联系人表
 * @author： gurui
 * @date：   2019-05-08
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdHospitalContacts")
@Slf4j
public class FdHospitalContactsController {
	@Autowired
	private IFdHospitalContactsService fdHospitalContactsService;
	
	/**
	  * 分页列表查询
	 * @param fdHospitalContacts
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdHospitalContacts>> queryPageList(FdHospitalContacts fdHospitalContacts,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdHospitalContacts>> result = new Result<IPage<FdHospitalContacts>>();
		//对医院联系人模糊查询
		if (StringUtils.isNotBlank(fdHospitalContacts.getContactPsn())){
			fdHospitalContacts.setContactPsn("*"+fdHospitalContacts.getContactPsn()+"*");
		}

		QueryWrapper<FdHospitalContacts> queryWrapper = QueryGenerator.initQueryWrapper(fdHospitalContacts, req.getParameterMap());
		Page<FdHospitalContacts> page = new Page<FdHospitalContacts>(pageNo, pageSize);
		IPage<FdHospitalContacts> pageList = fdHospitalContactsService.page(page, queryWrapper);
		// IPage<FdHospitalContacts> pageList = fdHospitalContactsService.findByPage(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdHospitalContacts
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	public Result<FdHospitalContacts> add(@RequestBody @Valid FdHospitalContacts fdHospitalContacts, BindingResult bindingResult) {
		Result<FdHospitalContacts> result = new Result<FdHospitalContacts>();
		try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()){
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
			fdHospitalContacts.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
			fdHospitalContactsService.save(fdHospitalContacts);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败:"+e.getMessage());

		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdHospitalContacts
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	public Result<FdHospitalContacts> edit(@RequestBody @Valid FdHospitalContacts fdHospitalContacts) {
		Result<FdHospitalContacts> result = new Result<FdHospitalContacts>();
		FdHospitalContacts fdHospitalContactsEntity = fdHospitalContactsService.getById(fdHospitalContacts.getId());
		if(fdHospitalContactsEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdHospitalContactsService.updateById(fdHospitalContacts);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdHospitalContacts> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdHospitalContacts> result = new Result<FdHospitalContacts>();
		FdHospitalContacts fdHospitalContacts = fdHospitalContactsService.getById(id);
		if(fdHospitalContacts==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdHospitalContactsService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdHospitalContacts> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdHospitalContacts> result = new Result<FdHospitalContacts>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdHospitalContactsService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdHospitalContacts> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdHospitalContacts> result = new Result<FdHospitalContacts>();
		FdHospitalContacts fdHospitalContacts = fdHospitalContactsService.getById(id);
		if(fdHospitalContacts==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdHospitalContacts);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdHospitalContacts> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdHospitalContacts fdHospitalContacts = JSON.parseObject(deString, FdHospitalContacts.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdHospitalContacts, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdHospitalContacts> pageList = fdHospitalContactsService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "医院联系人表列表");
      mv.addObject(NormalExcelConstants.CLASS, FdHospitalContacts.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("医院联系人表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdHospitalContacts> listFdHospitalContactss = ExcelImportUtil.importExcel(file.getInputStream(), FdHospitalContacts.class, params);
              for (FdHospitalContacts fdHospitalContactsExcel : listFdHospitalContactss) {
                  fdHospitalContactsService.save(fdHospitalContactsExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdHospitalContactss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
