package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdHospital;
import org.jeecg.modules.basedata.entity.FdHospitalContacts;
import org.jeecg.modules.basedata.service.IFdHospitalContactsService;
import org.jeecg.modules.basedata.service.IFdHospitalService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 医院档案表
 * @author： jeecg-boot
 * @date： 2019-05-05
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdHospital")
@Slf4j
public class FdHospitalController {
    @Autowired
    private IFdHospitalService fdHospitalService;
    @Autowired
    private IFdHospitalContactsService iFdHospitalContactsService;

    /**
     * 分页列表查询
     *
     * @param fdHospital
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdHospital>> queryPageList(FdHospital fdHospital,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest req) {
        Result<IPage<FdHospital>> result = new Result<IPage<FdHospital>>();
        //对医院名称模糊查询
        String name=fdHospital.getName();
        if(fdHospital.getName()!=null&&fdHospital.getName()!=""){
            fdHospital.setName(null);
        }
        String code = fdHospital.getCode();
         if(StringUtils.isNotBlank(fdHospital.getCode())){
             fdHospital.setName(null);
         }
        QueryWrapper<FdHospital> queryWrapper = QueryGenerator.initQueryWrapper(fdHospital, req.getParameterMap());

        if(name!=null&&name!=""){
            queryWrapper.like("fh.name",name);
        }
        if(code!=null&&code!=""){
            queryWrapper.like("fh.code",code);
        }
        queryWrapper.eq("fhc.is_default","1");
        //连表查询，只带出默认联系人
        queryWrapper.orderByDesc("fh.code");

        Page<FdHospital> page = new Page<FdHospital>(pageNo, pageSize);
        //IPage<FdHospital> pageList = fdHospitalService.page(page, queryWrapper);
        IPage<FdHospital> pageList = fdHospitalService.findByPage(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询
     *
     * @param fdHospital
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list1")
    @RequiresPermissions("url:hospital:view")
    public Result<IPage<FdHospital>> queryPageList1(FdHospital fdHospital,
                                                   @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                   @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                   HttpServletRequest req) {
        Result<IPage<FdHospital>> result = new Result<IPage<FdHospital>>();
        //对医院名称模糊查询
        String name=fdHospital.getName();
        if(fdHospital.getName()!=null&&fdHospital.getName()!=""){
            fdHospital.setName(null);
        }
        String code = fdHospital.getCode();
        if(StringUtils.isNotBlank(fdHospital.getCode())){
            fdHospital.setName(null);
        }
        QueryWrapper<FdHospital> queryWrapper = QueryGenerator.initQueryWrapper(fdHospital, req.getParameterMap());

        if(name!=null&&name!=""){
            queryWrapper.like("fh.name",name);
        }
        if(code!=null&&code!=""){
            queryWrapper.like("fh.code",code);
        }
        queryWrapper.eq("fhc.is_default","1");
        //连表查询，只带出默认联系人
        queryWrapper.orderByDesc("fh.code");

        Page<FdHospital> page = new Page<FdHospital>(pageNo, pageSize);
        //IPage<FdHospital> pageList = fdHospitalService.page(page, queryWrapper);
        IPage<FdHospital> pageList = fdHospitalService.findByPage(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdHospital
     * @return
     */
    @PostMapping(value = "/add")
    @NotDuplicate
    @RequiresPermissions(value = {"hospital:add","url:pm-quotation:view","url:pm-macromolecule:view","url:pm-projectArchives:view"},logical = Logical.OR)
    public Result<FdHospital> add(@RequestBody @Valid FdHospital fdHospital , BindingResult bindingResult) {
        Result<FdHospital> result = new Result<FdHospital>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()){
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            //fdHospitalService.save(fdHospital);
            fdHospitalService.saveHospitalAndContacts(fdHospital);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败:"+e.getMessage());
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdHospital
     * @return
     */
    @PutMapping(value = "/edit")
    @NotDuplicate
    @RequiresPermissions("hospital:update")
    public Result<FdHospital> edit(@RequestBody FdHospital fdHospital) {
        Result<FdHospital> result = new Result<FdHospital>();
        FdHospital fdHospitalEntity = fdHospitalService.getById(fdHospital.getId());
        if (fdHospitalEntity == null) {
            result.error500("未找到对应实体");
        } else {
            if(!StringUtils.isNotBlank(fdHospital.getCode())){
                result.error500("医院编码不能为空");
                return result;
            }
            if(!StringUtils.isNotBlank(fdHospital.getName())){
                result.error500("医院不能为空");
                return result;
            }
            // List<FdHospitalContacts> byHospitalId = iFdHospitalContactsService.getByHospitalId(fdHospital.getId());
//            if(byHospitalId==null || byHospitalId.isEmpty()){
//                result.error500("至少需要一个默认联系人");
//                return result;
//            }
//             int isDefaultCount = 0;
//             for(FdHospitalContacts fdHospitalContacts:byHospitalId){
//                 if("1".equals(fdHospitalContacts.getIsDefault())){
//                     isDefaultCount++;
//                 }
//             }
//            if(isDefaultCount==0){
//                result.error500("请至少指定一个默认联系人");
//                return result;
//            }
//            if(isDefaultCount>1){
//                result.error500("最多只能有一个默认联系人");
//                return result;
//            }
            boolean ok = fdHospitalService.updateById(fdHospital);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    @NotDuplicate
    @RequiresPermissions("hospital:update")
    public Result<FdHospital> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdHospital> result = new Result<FdHospital>();
        FdHospital fdHospital = fdHospitalService.getById(id);
        if (fdHospital == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdHospitalService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdHospital> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdHospital> result = new Result<FdHospital>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdHospitalService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdHospital> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdHospital> result = new Result<FdHospital>();
        FdHospital fdHospital = fdHospitalService.getById(id);
        if (fdHospital == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdHospital);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdHospital> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdHospital fdHospital = JSON.parseObject(deString, FdHospital.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdHospital, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdHospital> pageList = fdHospitalService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "医院档案表列表");
        mv.addObject(NormalExcelConstants.CLASS, FdHospital.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("医院档案表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdHospital> listFdHospitals = ExcelImportUtil.importExcel(file.getInputStream(), FdHospital.class, params);
                for (FdHospital fdHospitalExcel : listFdHospitals) {
                    fdHospitalService.save(fdHospitalExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdHospitals.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
