package org.jeecg.modules.basedata.controller;

import java.util.*;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdMatrixType;
import org.jeecg.modules.basedata.service.IFdMatrixTypeService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 基质分类
 * @author： jeecg-boot
 * @date：   2019-05-05
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdMatrixType")
@Slf4j
public class FdMatrixTypeController {
	@Autowired
	private IFdMatrixTypeService fdMatrixTypeService;
	
	/**
	  * 分页列表查询
	 * @param fdMatrixType
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdMatrixType>> queryPageList(FdMatrixType fdMatrixType,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdMatrixType>> result = new Result<IPage<FdMatrixType>>();
		//编码
		String code = fdMatrixType.getCode();
		if (StringUtils.isNotBlank(fdMatrixType.getCode())) {
			fdMatrixType.setCode(null);
		}
		//名称
		String name = fdMatrixType.getName();
		if (StringUtils.isNotBlank(fdMatrixType.getName())) {
			fdMatrixType.setName(null);
		}
		fdMatrixType.setDelFlag(CommonConstant.DEL_FLAG_0.toString().toString());
		QueryWrapper<FdMatrixType> queryWrapper = QueryGenerator.initQueryWrapper(fdMatrixType, req.getParameterMap());
		if (StringUtils.isNotBlank(code)) {
			queryWrapper.like("code", code);
		}
		if (StringUtils.isNotBlank(name)) {
			queryWrapper.like("name", name);
		}
		queryWrapper.lambda().orderBy(true,true,FdMatrixType::getCode);
		Page<FdMatrixType> page = new Page<FdMatrixType>(pageNo, pageSize);
		IPage<FdMatrixType> pageList = fdMatrixTypeService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	 /**
	  * 分页列表查询
	  * @param fdMatrixType
	  * @param pageNo
	  * @param pageSize
	  * @param req
	  * @return
	  */
	 @GetMapping(value = "/list1")
	 @RequiresPermissions("url:matrix-type:view")
	 public Result<IPage<FdMatrixType>> queryPageList1(FdMatrixType fdMatrixType,
													  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
													  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
													  HttpServletRequest req) {
		 Result<IPage<FdMatrixType>> result = new Result<IPage<FdMatrixType>>();
		 //编码
		 String code = fdMatrixType.getCode();
		 if (StringUtils.isNotBlank(fdMatrixType.getCode())) {
			 fdMatrixType.setCode(null);
		 }
		 //名称
		 String name = fdMatrixType.getName();
		 if (StringUtils.isNotBlank(fdMatrixType.getName())) {
			 fdMatrixType.setName(null);
		 }
		 fdMatrixType.setDelFlag(CommonConstant.DEL_FLAG_0.toString().toString());
		 QueryWrapper<FdMatrixType> queryWrapper = QueryGenerator.initQueryWrapper(fdMatrixType, req.getParameterMap());
		 if (StringUtils.isNotBlank(code)) {
			 queryWrapper.like("code", code);
		 }
		 if (StringUtils.isNotBlank(name)) {
			 queryWrapper.like("name", name);
		 }
		 queryWrapper.lambda().orderBy(true,true,FdMatrixType::getCode);
		 Page<FdMatrixType> page = new Page<FdMatrixType>(pageNo, pageSize);
		 IPage<FdMatrixType> pageList = fdMatrixTypeService.page(page, queryWrapper);
		 result.setSuccess(true);
		 result.setResult(pageList);
		 return result;
	 }
	
	/**
	  *   添加
	 * @param fdMatrixType
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	@RequiresPermissions("matrix-type:add")
	public Result<FdMatrixType> add(@RequestBody @Valid FdMatrixType fdMatrixType, BindingResult bindingResult) {
		Result<FdMatrixType> result = new Result<FdMatrixType>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
		    fdMatrixType.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
			fdMatrixTypeService.save(fdMatrixType);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败:"+e.getMessage());

		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdMatrixType
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	@RequiresPermissions("matrix-type:update")
	public Result<FdMatrixType> edit(@RequestBody @Valid FdMatrixType fdMatrixType, BindingResult bindingResult) {
		Result<FdMatrixType> result = new Result<FdMatrixType>();
		FdMatrixType fdMatrixTypeEntity = fdMatrixTypeService.getById(fdMatrixType.getId());
		if(fdMatrixTypeEntity==null) {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdMatrixTypeService.updateById(fdMatrixType);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}

	 /**
	  *   通过id删除
	  * @param id
	  * @return
	  */
	 @DeleteMapping(value = "/delete")
	 public Result<FdMatrixType> delete(@RequestParam(name="id",required=true) String id) {
		 Result<FdMatrixType> result = new Result<FdMatrixType>();
		 FdMatrixType fdMatrixType = fdMatrixTypeService.getById(id);
		 if(fdMatrixType==null) {
			 result.error500("未找到对应实体");
		 }else {
		 	 fdMatrixType.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
			 boolean ok = fdMatrixTypeService.updateById(fdMatrixType);
			 if(ok) {
				 result.success("删除成功!");
			 }
		 }

		 return result;
	 }
	

	/**
	 * @Author 谭吉
	 * @Description //TODO	单个或多个删除
	 * @Date 14:07 2019/5/13
	 * @Param [ids]
	 * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.basedata.entity.FdMatrixType>
	 **/
	@DeleteMapping(value = "/deleteBatch")
	@NotDuplicate
	@RequiresPermissions("matrix-type:delete")
	public Result<FdMatrixType> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdMatrixType> result = new Result<FdMatrixType>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			List list = Arrays.asList(ids.split(","));
			FdMatrixType fdMatrixType = null;
			Collection<FdMatrixType> collectionList = new ArrayList<>();
			for (int i = 0; i < list.size(); i++){
				fdMatrixType=new FdMatrixType();
				fdMatrixType.setId(list.get(i).toString());
				collectionList.add(fdMatrixType);
			}
			for ( FdMatrixType fdMatrixTypes : collectionList) {
				fdMatrixType.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
				fdMatrixTypeService.updateById(fdMatrixTypes);
			}
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdMatrixType> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdMatrixType> result = new Result<FdMatrixType>();
		FdMatrixType fdMatrixType = fdMatrixTypeService.getById(id);
		if(fdMatrixType==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdMatrixType);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdMatrixType> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdMatrixType fdMatrixType = JSON.parseObject(deString, FdMatrixType.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdMatrixType, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdMatrixType> pageList = fdMatrixTypeService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "基质分类列表");
      mv.addObject(NormalExcelConstants.CLASS, FdMatrixType.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("基质分类列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdMatrixType> listFdMatrixTypes = ExcelImportUtil.importExcel(file.getInputStream(), FdMatrixType.class, params);
              for (FdMatrixType fdMatrixTypeExcel : listFdMatrixTypes) {
                  fdMatrixTypeService.save(fdMatrixTypeExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdMatrixTypes.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
