package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdMilestoneTemplate;
import org.jeecg.modules.basedata.entity.FdPersonnel;
import org.springframework.validation.BindingResult;
import org.jeecg.modules.basedata.service.IFdMilestoneTemplateService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 里程碑模板
 * @author： jeecg-boot
 * @date： 2019-05-14
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdMilestoneTemplate")
@Slf4j
public class FdMilestoneTemplateController {
    @Autowired
    private IFdMilestoneTemplateService fdMilestoneTemplateService;

    /**
     * 分页列表查询
     *
     * @param fdMilestoneTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdMilestoneTemplate>> queryPageList(FdMilestoneTemplate fdMilestoneTemplate,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<FdMilestoneTemplate>> result = new Result<IPage<FdMilestoneTemplate>>();
        String name = fdMilestoneTemplate.getName();
        if (StringUtils.isNotBlank(fdMilestoneTemplate.getName())) {
            fdMilestoneTemplate.setName(null);
        }
        String description = fdMilestoneTemplate.getDescription();
        if (StringUtils.isNotBlank(fdMilestoneTemplate.getDescription())) {
            fdMilestoneTemplate.setDescription(null);
        }
        String type=fdMilestoneTemplate.getType();
        if(StringUtils.isNotBlank(type)){
            fdMilestoneTemplate.setType(null);
        }
        QueryWrapper<FdMilestoneTemplate> queryWrapper = QueryGenerator.initQueryWrapper(fdMilestoneTemplate, req.getParameterMap());
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("name", name);
        }
        if (StringUtils.isNotBlank(description)) {
            queryWrapper.like("description", description);
        }
        if (StringUtils.isNotBlank(type)&&!type.equals("null")) {
            queryWrapper.like("type", type);
        }
        queryWrapper.lambda().orderByDesc(FdMilestoneTemplate::getCreateTime);
        Page<FdMilestoneTemplate> page = new Page<FdMilestoneTemplate>(pageNo, pageSize);
        IPage<FdMilestoneTemplate> pageList = fdMilestoneTemplateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 分页列表查询（有权限）
     *
     * @param fdMilestoneTemplate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list1")
    @RequiresPermissions("url:milestone:view")
    public Result<IPage<FdMilestoneTemplate>> queryPageList1(FdMilestoneTemplate fdMilestoneTemplate,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<FdMilestoneTemplate>> result = new Result<IPage<FdMilestoneTemplate>>();
        String name = fdMilestoneTemplate.getName();
        if (StringUtils.isNotBlank(fdMilestoneTemplate.getName())) {
            fdMilestoneTemplate.setName(null);
        }
        String description = fdMilestoneTemplate.getDescription();
        if (StringUtils.isNotBlank(fdMilestoneTemplate.getDescription())) {
            fdMilestoneTemplate.setDescription(null);
        }
        String type=fdMilestoneTemplate.getType();
        if(type!=null&&type.equals("null")){
            fdMilestoneTemplate.setType(null);
        }
        QueryWrapper<FdMilestoneTemplate> queryWrapper = QueryGenerator.initQueryWrapper(fdMilestoneTemplate, req.getParameterMap());
        if (StringUtils.isNotBlank(name)) {
            queryWrapper.like("name", name);
        }
        if (StringUtils.isNotBlank(description)) {
            queryWrapper.like("description", description);
        }
        queryWrapper.lambda().orderByDesc(FdMilestoneTemplate::getCreateTime);
        Page<FdMilestoneTemplate> page = new Page<FdMilestoneTemplate>(pageNo, pageSize);
        IPage<FdMilestoneTemplate> pageList = fdMilestoneTemplateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdMilestoneTemplate
     * @return
     */
    @PostMapping(value = "/add")
    @NotDuplicate
    @RequiresPermissions("milestone:add")
    public Result<FdMilestoneTemplate> add(@RequestBody @Valid FdMilestoneTemplate fdMilestoneTemplate,BindingResult bindingResult) {
        Result<FdMilestoneTemplate> result = new Result<FdMilestoneTemplate>();
        try {
            StringBuilder stringBuilder=new StringBuilder();
            if(bindingResult.hasErrors()){
                //实体错误信息
                bindingResult.getAllErrors().stream().forEach(error -> stringBuilder.append(error.getDefaultMessage() + "<br/>"));
                result.error500(stringBuilder.toString());
                return result;
            }
            fdMilestoneTemplate.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
            fdMilestoneTemplateService.save(fdMilestoneTemplate);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败:"+e.getMessage());

        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdMilestoneTemplate
     * @return
     */
    @PutMapping(value = "/edit")
    @NotDuplicate
    @RequiresPermissions("milestone:update")
    public Result<FdMilestoneTemplate> edit(@RequestBody @Valid FdMilestoneTemplate fdMilestoneTemplate,BindingResult bindingResult) {
        Result<FdMilestoneTemplate> result = new Result<FdMilestoneTemplate>();
        FdMilestoneTemplate fdMilestoneTemplateEntity = fdMilestoneTemplateService.getById(fdMilestoneTemplate.getId());
        if (fdMilestoneTemplateEntity == null) {
            StringBuilder stringBuilder=new StringBuilder();
            if(bindingResult.hasErrors()){
                //实体错误信息
                bindingResult.getAllErrors().stream().forEach(error -> stringBuilder.append(error.getDefaultMessage() + "<br/>"));
                result.error500(stringBuilder.toString());
                return result;
            }
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdMilestoneTemplateService.updateById(fdMilestoneTemplate);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    @NotDuplicate
    @RequiresPermissions("milestone:delete")
    public Result<FdMilestoneTemplate> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdMilestoneTemplate> result = new Result<FdMilestoneTemplate>();
        FdMilestoneTemplate fdMilestoneTemplate = fdMilestoneTemplateService.getById(id);
        if (fdMilestoneTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdMilestoneTemplateService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdMilestoneTemplate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdMilestoneTemplate> result = new Result<FdMilestoneTemplate>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdMilestoneTemplateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdMilestoneTemplate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdMilestoneTemplate> result = new Result<FdMilestoneTemplate>();
        FdMilestoneTemplate fdMilestoneTemplate = fdMilestoneTemplateService.getById(id);
        if (fdMilestoneTemplate == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdMilestoneTemplate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdMilestoneTemplate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdMilestoneTemplate fdMilestoneTemplate = JSON.parseObject(deString, FdMilestoneTemplate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdMilestoneTemplate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdMilestoneTemplate> pageList = fdMilestoneTemplateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "里程碑模板列表");
        mv.addObject(NormalExcelConstants.CLASS, FdMilestoneTemplate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("里程碑模板列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdMilestoneTemplate> listFdMilestoneTemplates = ExcelImportUtil.importExcel(file.getInputStream(), FdMilestoneTemplate.class, params);
                for (FdMilestoneTemplate fdMilestoneTemplateExcel : listFdMilestoneTemplates) {
                    fdMilestoneTemplateService.save(fdMilestoneTemplateExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdMilestoneTemplates.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }


    /**
     * @return org.jeecg.common.api.vo.Result<java.util.Map < java.lang.String, java.lang.Object>>
     * @Author gurui
     * @Description //TODO 分页查询数据
     * @Date 15:40 2019/5/14
     * @Param [fdMilestoneTemplate, pageNo, pageSize, req]
     **/
    @GetMapping(value = "/listAllPage")
    public Result<Map<String, Object>> getListOpsition(FdMilestoneTemplate fdMilestoneTemplate,
                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                       HttpServletRequest req) {
        Result<Map<String, Object>> result = new Result<>();
        Map<String, Object> map = Maps.newHashMap();//limit参数
        map.put("fdMilestoneTemplate", fdMilestoneTemplate);
        map.put("current", pageNo);//当前页
        map.put("size", pageSize);//每页显示
        Map<String, Object> list = null;
        try {
            list = fdMilestoneTemplateService.getPageFdMilestoneTemplate(map);
            result.setSuccess(true);
            result.setMessage("查询成功");
        } catch (Exception e) {
            e.printStackTrace();
            result.setSuccess(false);
            result.setMessage("未查到数据");
        }
        result.setResult(list);
        return result;
    }

    /**
     * @Author gurui
     * @Description //TODO 复制一条一样的模板
     * @Date 16:44 2019/7/5
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.basedata.entity.FdMilestoneTemplate>
     **/
    @PostMapping(value = "/copy")
    @NotDuplicate
    public Result<FdMilestoneTemplate> copy(@RequestBody String id) {
        Result<FdMilestoneTemplate> result = new Result<FdMilestoneTemplate>();
        try {
            fdMilestoneTemplateService.copy(id);
            result.success("复制成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败"+e.getMessage());
        }
        return result;
    }

}
