package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdPassFeeClassProject;
import org.jeecg.modules.basedata.service.IFdPassFeeClassProjectService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 过手费项目档案
 * @author： jeecg-boot
 * @date：   2019-05-10
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdPassFeeClassProject")
@Slf4j
public class FdPassFeeClassProjectController {
	@Autowired
	private IFdPassFeeClassProjectService fdPassFeeClassProjectService;
	
	/**
	  * 分页列表查询
	 * @param fdPassFeeClassProject
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdPassFeeClassProject>> queryPageList(FdPassFeeClassProject fdPassFeeClassProject,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdPassFeeClassProject>> result = new Result<IPage<FdPassFeeClassProject>>();
		//实体查询条件
		String name=fdPassFeeClassProject.getName();
		String code=fdPassFeeClassProject.getCode();
		//设空
		if(StringUtils.isNotBlank(fdPassFeeClassProject.getName())){
			fdPassFeeClassProject.setName(null);
		}
		if(StringUtils.isNotBlank(fdPassFeeClassProject.getCode())){
			fdPassFeeClassProject.setCode(null);
		}
		QueryWrapper<FdPassFeeClassProject> queryWrapper = QueryGenerator.initQueryWrapper(fdPassFeeClassProject, req.getParameterMap());
		//重置条件sql
		if(StringUtils.isNotBlank(name)){
			queryWrapper.like("pro.name",name);
		}
		if(StringUtils.isNotBlank(code)){
			queryWrapper.eq("pro.code",code);
		}
		queryWrapper.eq("pro.del_flag",0).eq("cla.del_flag",0);//查询条件（不是删除状态）
		Page<FdPassFeeClassProject> page = new Page<FdPassFeeClassProject>(pageNo, pageSize);
		try {
			IPage<FdPassFeeClassProject> pageList = fdPassFeeClassProjectService.getListByPage(page, queryWrapper);
			result.setSuccess(true);
			result.setResult(pageList);
		} catch (Exception e) {
			result.setSuccess(false);
			e.printStackTrace();
		}
		return result;
	}


	 /**
	  * 分页列表查询（有权限）
	  * @param fdPassFeeClassProject
	  * @param pageNo
	  * @param pageSize
	  * @param req
	  * @return
	  */
	 @GetMapping(value = "/list1")
	 @RequiresPermissions("url:cost:view")
	 public Result<IPage<FdPassFeeClassProject>> queryPageList1(FdPassFeeClassProject fdPassFeeClassProject,
															   @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
															   @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
															   HttpServletRequest req) {
		 Result<IPage<FdPassFeeClassProject>> result = new Result<IPage<FdPassFeeClassProject>>();
		 //实体查询条件
		 String name=fdPassFeeClassProject.getName();
		 String code=fdPassFeeClassProject.getCode();
		 //设空
		 if(StringUtils.isNotBlank(fdPassFeeClassProject.getName())){
			 fdPassFeeClassProject.setName(null);
		 }
		 if(StringUtils.isNotBlank(fdPassFeeClassProject.getCode())){
			 fdPassFeeClassProject.setCode(null);
		 }
		 QueryWrapper<FdPassFeeClassProject> queryWrapper = QueryGenerator.initQueryWrapper(fdPassFeeClassProject, req.getParameterMap());
		 //重置条件sql
		 if(StringUtils.isNotBlank(name)){
			 queryWrapper.like("pro.name",name);
		 }
		 if(StringUtils.isNotBlank(code)){
			 queryWrapper.eq("pro.code",code);
		 }
		 queryWrapper.eq("pro.del_flag",0).eq("cla.del_flag",0);//查询条件（不是删除状态）
		 Page<FdPassFeeClassProject> page = new Page<FdPassFeeClassProject>(pageNo, pageSize);
		 try {
			 IPage<FdPassFeeClassProject> pageList = fdPassFeeClassProjectService.getListByPage(page, queryWrapper);
			 result.setSuccess(true);
			 result.setResult(pageList);
		 } catch (Exception e) {
			 result.setSuccess(false);
			 e.printStackTrace();
		 }
		 return result;
	 }
	
	/**
	  *   添加
	 * @param fdPassFeeClassProject
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	@RequiresPermissions("cost-item:add")
	public Result<FdPassFeeClassProject> add(@RequestBody @Valid FdPassFeeClassProject fdPassFeeClassProject,BindingResult bindingResult) {
		Result<FdPassFeeClassProject> result = new Result<FdPassFeeClassProject>();
		try {
			StringBuilder stringBuilder=new StringBuilder();
			if(bindingResult.hasErrors()){
				//实体错误信息
				bindingResult.getAllErrors().stream().forEach(error -> stringBuilder.append(error.getDefaultMessage() + "<br/>"));
				result.error500(stringBuilder.toString());
				return result;
			}
			fdPassFeeClassProject.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
			fdPassFeeClassProjectService.save(fdPassFeeClassProject);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败:"+e.getMessage());

		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdPassFeeClassProject
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	@RequiresPermissions("cost-item:update")
	public Result<FdPassFeeClassProject> edit(@RequestBody @Valid FdPassFeeClassProject fdPassFeeClassProject, BindingResult bindingResult) {
		Result<FdPassFeeClassProject> result = new Result<FdPassFeeClassProject>();
		FdPassFeeClassProject fdPassFeeClassProjectEntity = fdPassFeeClassProjectService.getById(fdPassFeeClassProject.getId());
		if(fdPassFeeClassProjectEntity==null) {
			StringBuilder stringBuilder=new StringBuilder();
			if(bindingResult.hasErrors()){
				//实体错误信息
				bindingResult.getAllErrors().stream().forEach(error -> stringBuilder.append(error.getDefaultMessage() + "<br/>"));
				result.error500(stringBuilder.toString());
				return result;
			}
			result.error500("未找到对应实体");
		}else {
			String pkPassFeeClass = fdPassFeeClassProject.getPkPassFeeClass();
			if (pkPassFeeClass == null || "".equals(pkPassFeeClass) || "null".equals(pkPassFeeClass)){
				result.error500("过收费类型不能为空");
				return result;
			}
			boolean ok = fdPassFeeClassProjectService.updateById(fdPassFeeClassProject);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	@NotDuplicate
	@RequiresPermissions("cost-item:delete")
	public Result<FdPassFeeClassProject> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdPassFeeClassProject> result = new Result<FdPassFeeClassProject>();
		FdPassFeeClassProject fdPassFeeClassProject = fdPassFeeClassProjectService.getById(id);
		if(fdPassFeeClassProject==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdPassFeeClassProjectService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdPassFeeClassProject> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdPassFeeClassProject> result = new Result<FdPassFeeClassProject>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdPassFeeClassProjectService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdPassFeeClassProject> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdPassFeeClassProject> result = new Result<FdPassFeeClassProject>();
		FdPassFeeClassProject fdPassFeeClassProject = fdPassFeeClassProjectService.getById(id);
		if(fdPassFeeClassProject==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdPassFeeClassProject);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdPassFeeClassProject> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdPassFeeClassProject fdPassFeeClassProject = JSON.parseObject(deString, FdPassFeeClassProject.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdPassFeeClassProject, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdPassFeeClassProject> pageList = fdPassFeeClassProjectService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "过手费项目档案列表");
      mv.addObject(NormalExcelConstants.CLASS, FdPassFeeClassProject.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("过手费项目档案列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdPassFeeClassProject> listFdPassFeeClassProjects = ExcelImportUtil.importExcel(file.getInputStream(), FdPassFeeClassProject.class, params);
              for (FdPassFeeClassProject fdPassFeeClassProjectExcel : listFdPassFeeClassProjects) {
                  fdPassFeeClassProjectService.save(fdPassFeeClassProjectExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdPassFeeClassProjects.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
