package org.jeecg.modules.basedata.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdStoragePlace;
import org.jeecg.modules.basedata.service.IFdStoragePlaceService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 存储位置表
 * @author： gurui
 * @date：   2019-05-07
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdStoragePlace")
@Slf4j
public class FdStoragePlaceController {
	@Autowired
	private IFdStoragePlaceService fdStoragePlaceService;
	
	/**
	  * 分页列表查询
	 * @param fdStoragePlace
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdStoragePlace>> queryPageList(FdStoragePlace fdStoragePlace,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdStoragePlace>> result = new Result<IPage<FdStoragePlace>>();
		//对存储位置编码进行模糊查询
		 String code = fdStoragePlace.getCode();
		if (StringUtils.isNotBlank(fdStoragePlace.getCode())){
			fdStoragePlace.setCode(fdStoragePlace.getCode());
		}
		QueryWrapper<FdStoragePlace> queryWrapper = QueryGenerator.initQueryWrapper(fdStoragePlace, req.getParameterMap());
		if (StringUtils.isNotBlank(code)) {
			queryWrapper.like("code", code);
		}
		queryWrapper.lambda().orderByDesc(FdStoragePlace::getCreateTime);
		Page<FdStoragePlace> page = new Page<FdStoragePlace>(pageNo, pageSize);
		IPage<FdStoragePlace> pageList = fdStoragePlaceService.page(page, queryWrapper);
		// IPage<FdStoragePlace> pageList = fdStoragePlaceService.findByPage(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}


	 /**
	  * 分页列表查询
	  * @param fdStoragePlace
	  * @param pageNo
	  * @param pageSize
	  * @param req
	  * @return
	  */
	 @GetMapping(value = "/list1")
	 @RequiresPermissions("url:store:view")
	 public Result<IPage<FdStoragePlace>> queryPageList1(FdStoragePlace fdStoragePlace,
														@RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
														@RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
														HttpServletRequest req) {
		 Result<IPage<FdStoragePlace>> result = new Result<IPage<FdStoragePlace>>();
		 //对存储位置编码进行模糊查询
		 String code = fdStoragePlace.getCode();
		 if (StringUtils.isNotBlank(fdStoragePlace.getCode())){
			 fdStoragePlace.setCode(fdStoragePlace.getCode());
		 }
		 QueryWrapper<FdStoragePlace> queryWrapper = QueryGenerator.initQueryWrapper(fdStoragePlace, req.getParameterMap());
		 if (StringUtils.isNotBlank(code)) {
			 queryWrapper.like("code", code);
		 }
		 queryWrapper.lambda().orderByDesc(FdStoragePlace::getCreateTime);
		 Page<FdStoragePlace> page = new Page<FdStoragePlace>(pageNo, pageSize);
		 IPage<FdStoragePlace> pageList = fdStoragePlaceService.page(page, queryWrapper);
		 // IPage<FdStoragePlace> pageList = fdStoragePlaceService.findByPage(page, queryWrapper);
		 result.setSuccess(true);
		 result.setResult(pageList);
		 return result;
	 }
	
	/**
	  *   添加
	 * @param fdStoragePlace
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	@RequiresPermissions("store:add")
	public Result<FdStoragePlace> add(@RequestBody @Valid FdStoragePlace fdStoragePlace, BindingResult bindingResult) {
		Result<FdStoragePlace> result = new Result<FdStoragePlace>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}

			//查询是否存在相同编码
			LambdaQueryWrapper<FdStoragePlace> queryWrapper=new LambdaQueryWrapper<>();
			queryWrapper.eq(FdStoragePlace::getCode,fdStoragePlace.getCode());
			queryWrapper.eq(FdStoragePlace::getDelFlag,"0");
			List<FdStoragePlace> fdStoragePlaceList=fdStoragePlaceService.list(queryWrapper);
			if(!fdStoragePlaceList.isEmpty()&&fdStoragePlaceList.size()>0){
				result.error500("编码已存在");
				return result;
			}
			//fdStoragePlace.setEnable("0");
			fdStoragePlace.setDelFlag(CommonConstant.DEL_FLAG_0.toString());
			fdStoragePlaceService.save(fdStoragePlace);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败:"+e.getMessage());
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdStoragePlace
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	@RequiresPermissions("store:update")
	public Result<FdStoragePlace> edit(@RequestBody @Valid FdStoragePlace fdStoragePlace) {
		Result<FdStoragePlace> result = new Result<FdStoragePlace>();
		FdStoragePlace fdStoragePlaceEntity = fdStoragePlaceService.getById(fdStoragePlace.getId());
		if(fdStoragePlaceEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdStoragePlaceService.updateById(fdStoragePlace);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	@NotDuplicate
	@RequiresPermissions("store:delete")
	public Result<FdStoragePlace> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdStoragePlace> result = new Result<FdStoragePlace>();
		FdStoragePlace fdStoragePlace = fdStoragePlaceService.getById(id);
		if(fdStoragePlace==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdStoragePlaceService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdStoragePlace> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdStoragePlace> result = new Result<FdStoragePlace>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdStoragePlaceService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdStoragePlace> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdStoragePlace> result = new Result<FdStoragePlace>();
		FdStoragePlace fdStoragePlace = fdStoragePlaceService.getById(id);
		if(fdStoragePlace==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdStoragePlace);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdStoragePlace> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdStoragePlace fdStoragePlace = JSON.parseObject(deString, FdStoragePlace.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdStoragePlace, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdStoragePlace> pageList = fdStoragePlaceService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "存储位置表列表");
      mv.addObject(NormalExcelConstants.CLASS, FdStoragePlace.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("存储位置表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdStoragePlace> listFdStoragePlaces = ExcelImportUtil.importExcel(file.getInputStream(), FdStoragePlace.class, params);
              for (FdStoragePlace fdStoragePlaceExcel : listFdStoragePlaces) {
                  fdStoragePlaceService.save(fdStoragePlaceExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdStoragePlaces.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
