package org.jeecg.modules.basedata.controller;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdWorkTasks;
import org.jeecg.modules.basedata.entity.FdWorkTasksTree;
import org.jeecg.modules.basedata.service.IFdWorkTasksService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 工时任务
 * @author： jeecg-boot
 * @date：   2020-07-14
 * @version： V1.0
 */
@RestController
@RequestMapping("/basedata/fdWorkTasks")
@Slf4j
public class FdWorkTasksController {
	@Autowired
	private IFdWorkTasksService fdWorkTasksService;
	
	/**
	  * 分页列表查询
	 * @param fdWorkTasks
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdWorkTasks>> queryPageList(FdWorkTasks fdWorkTasks,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdWorkTasks>> result = new Result<IPage<FdWorkTasks>>();
		QueryWrapper<FdWorkTasks> queryWrapper = QueryGenerator.initQueryWrapper(fdWorkTasks, req.getParameterMap());
		Page<FdWorkTasks> page = new Page<FdWorkTasks>(pageNo, pageSize);
		IPage<FdWorkTasks> pageList = fdWorkTasksService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	 @GetMapping(value = "/getListTree")
	 public Result<List<FdWorkTasksTree>> getListTree(){
		 Result<List<FdWorkTasksTree>> result=new Result<List<FdWorkTasksTree>>();
		 try {
			 LambdaQueryWrapper<FdWorkTasks> lambdaQueryWrapper=new LambdaQueryWrapper<>();
			 lambdaQueryWrapper.eq(FdWorkTasks::getDelFlag,"0");
			 lambdaQueryWrapper.orderByAsc(FdWorkTasks::getSort);
			 List<FdWorkTasks> list = fdWorkTasksService.list(lambdaQueryWrapper);
			 List<FdWorkTasksTree> treeLists=new ArrayList<>();
			 getTreeList(treeLists,list,null);
			 findLeaf(treeLists);
			 result.setResult(treeLists);
		 }catch (Exception e){
			 e.fillInStackTrace();
		 }
		 return  result;
	 }

	 private void getTreeList(List<FdWorkTasksTree> treeList, List<FdWorkTasks> fdlist, FdWorkTasksTree temp) {
		 int i = 0;
		 for (FdWorkTasks fdWorkTasks : fdlist) {
			 String tempPid = fdWorkTasks.getParentId();
			 FdWorkTasksTree tree = new FdWorkTasksTree(fdWorkTasks);
			 if(temp == null) {
				 if(oConvertUtils.isEmpty(tempPid)){
					 treeList.add(tree);
				 }
				 if(!tree.getIsLeaf()) {
					 getTreeList(treeList, fdlist, tree);
				 }
			 }else if(tempPid != null && tempPid.equals(temp.getKey())){
				 temp.getChildren().add(tree);
				 if(!tree.getIsLeaf()) {
					 getTreeList(treeList, fdlist, tree);
				 }
			 }

		 }
	 }

	 private void findLeaf(List<FdWorkTasksTree> treeList){
		 for(FdWorkTasksTree fdWorkTasksTree : treeList){
			 if(fdWorkTasksTree.getChildren().isEmpty()){
				 fdWorkTasksTree.setIsLeaf(true);
			 }else{
				 findLeaf(fdWorkTasksTree.getChildren());
			 }
		 }

	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 根据查询条件查询工时任务
	 * @Date 2020/7/16 11:17
	 * @Param [fdWorkTasks]
	 * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.basedata.entity.FdWorkTasksTree>>
	 */
	 @GetMapping(value = "/getListTreeWhere")
	 public Result<List<FdWorkTasksTree>> getListTreeWhere(FdWorkTasks fdWorkTasks){
		 Result<List<FdWorkTasksTree>> result=new Result<List<FdWorkTasksTree>>();
		 try {
			 LambdaQueryWrapper<FdWorkTasks> lambdaQueryWrapper=new LambdaQueryWrapper<>();
			 lambdaQueryWrapper.eq(FdWorkTasks::getType,fdWorkTasks.getType());
			 lambdaQueryWrapper.eq(FdWorkTasks::getDelFlag,"0");
			 lambdaQueryWrapper.orderByAsc(FdWorkTasks::getSort);
			 List<FdWorkTasks> list = fdWorkTasksService.list(lambdaQueryWrapper);
			 List<FdWorkTasksTree> treeLists=new ArrayList<>();
			 getTreeList(treeLists,list,null);
			 findLeaf(treeLists);
			 result.setResult(treeLists);
		 }catch (Exception e){
			 e.fillInStackTrace();
		 }
		 return  result;
	 }
	
	/**
	  *   添加
	 * @param fdWorkTasks
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdWorkTasks> add(@RequestBody @Valid FdWorkTasks fdWorkTasks, BindingResult bindingResult) {
		Result<FdWorkTasks> result = new Result<FdWorkTasks>();

		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			fdWorkTasksService.save(fdWorkTasks);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdWorkTasks
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdWorkTasks> edit(@RequestBody @Valid FdWorkTasks fdWorkTasks, BindingResult bindingResult) {
		Result<FdWorkTasks> result = new Result<FdWorkTasks>();
		FdWorkTasks fdWorkTasksEntity = fdWorkTasksService.getById(fdWorkTasks.getId());
		if(fdWorkTasksEntity==null) {
			result.error500("未找到对应实体");
		}else {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()){
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			boolean ok = fdWorkTasksService.updateById(fdWorkTasks);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdWorkTasks> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdWorkTasks> result = new Result<FdWorkTasks>();
		FdWorkTasks fdWorkTasks = fdWorkTasksService.getById(id);
		if(fdWorkTasks==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdWorkTasksService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdWorkTasks> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdWorkTasks> result = new Result<FdWorkTasks>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdWorkTasksService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdWorkTasks> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdWorkTasks> result = new Result<FdWorkTasks>();
		FdWorkTasks fdWorkTasks = fdWorkTasksService.getById(id);
		if(fdWorkTasks==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdWorkTasks);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdWorkTasks> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdWorkTasks fdWorkTasks = JSON.parseObject(deString, FdWorkTasks.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdWorkTasks, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdWorkTasks> pageList = fdWorkTasksService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "工时任务列表");
      mv.addObject(NormalExcelConstants.CLASS, FdWorkTasks.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("工时任务列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdWorkTasks> listFdWorkTaskss = ExcelImportUtil.importExcel(file.getInputStream(), FdWorkTasks.class, params);
              for (FdWorkTasks fdWorkTasksExcel : listFdWorkTaskss) {
                  fdWorkTasksService.save(fdWorkTasksExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdWorkTaskss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }


	  @PostMapping(value ="/addAll" )
  public Result addAll(@RequestBody FdWorkTasks fdWorkTasks){

  	fdWorkTasksService.saveBatch(fdWorkTasks.getFdWorkTasksList());
	return new Result();
  }

}
