package org.jeecg.modules.fbsDispatchList.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.fbsDispatchList.entity.FbsDispatchList;
import org.jeecg.modules.fbsDispatchList.service.IFbsDispatchListService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 销售发货退货单主表（表头）
 * @author： jeecg-boot
 * @date：   2020-10-22
 * @version： V1.0
 */
@RestController
@RequestMapping("/fbsDispatchList/fbsDispatchList")
@Slf4j
public class FbsDispatchListController {
	@Autowired
	private IFbsDispatchListService fbsDispatchListService;

     /**
      * 发货通知(仓库管理看板)
      * @return
      */
     @RequestMapping(value = "/getWatingSendList")
     public Result<List<Map<String,Object>>> getWatingSendList(){
         Result<List<Map<String,Object>>> result=new Result<>();

         try {
             List<Map<String,Object>> map=fbsDispatchListService.getWatingSendList();
             result.setResult(map);
             result.success("操作成功");

         } catch (Exception e) {
             e.printStackTrace();
             result.error500("操作失败："+e.getMessage());
         }

         return result;
     }

     /**
      * 已入库数量(仓库管理看板)
      * @return
      */
     @RequestMapping(value = "/getSendSum")
     public int getSendSum(){
         return fbsDispatchListService.getSendSum();
     }






     /**
	  * 分页列表查询
	 * @param fbsDispatchList
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FbsDispatchList>> queryPageList(FbsDispatchList fbsDispatchList,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FbsDispatchList>> result = new Result<IPage<FbsDispatchList>>();
		QueryWrapper<FbsDispatchList> queryWrapper = QueryGenerator.initQueryWrapper(fbsDispatchList, req.getParameterMap());
		Page<FbsDispatchList> page = new Page<FbsDispatchList>(pageNo, pageSize);
		IPage<FbsDispatchList> pageList = fbsDispatchListService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fbsDispatchList
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FbsDispatchList> add(@RequestBody FbsDispatchList fbsDispatchList) {
		Result<FbsDispatchList> result = new Result<FbsDispatchList>();
		try {
			fbsDispatchListService.save(fbsDispatchList);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
//	/**
//	  *  编辑
//	 * @param fbsDispatchList
//	 * @return
//	 */
//	@PutMapping(value = "/edit")
//	public Result<FbsDispatchList> edit(@RequestBody FbsDispatchList fbsDispatchList) {
//		Result<FbsDispatchList> result = new Result<FbsDispatchList>();
//		FbsDispatchList fbsDispatchListEntity = fbsDispatchListService.getById(fbsDispatchList.getId());
//		if(fbsDispatchListEntity==null) {
//			result.error500("未找到对应实体");
//		}else {
//			boolean ok = fbsDispatchListService.updateById(fbsDispatchList);
//			//TODO 返回false说明什么？
//			if(ok) {
//				result.success("修改成功!");
//			}
//		}
//
//		return result;
//	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FbsDispatchList> delete(@RequestParam(name="id",required=true) String id) {
		Result<FbsDispatchList> result = new Result<FbsDispatchList>();
		FbsDispatchList fbsDispatchList = fbsDispatchListService.getById(id);
		if(fbsDispatchList==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsDispatchListService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FbsDispatchList> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FbsDispatchList> result = new Result<FbsDispatchList>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fbsDispatchListService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FbsDispatchList> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FbsDispatchList> result = new Result<FbsDispatchList>();
		FbsDispatchList fbsDispatchList = fbsDispatchListService.getById(id);
		if(fbsDispatchList==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fbsDispatchList);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FbsDispatchList> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FbsDispatchList fbsDispatchList = JSON.parseObject(deString, FbsDispatchList.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fbsDispatchList, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FbsDispatchList> pageList = fbsDispatchListService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "销售发货退货单主表（表头）列表");
      mv.addObject(NormalExcelConstants.CLASS, FbsDispatchList.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("销售发货退货单主表（表头）列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FbsDispatchList> listFbsDispatchLists = ExcelImportUtil.importExcel(file.getInputStream(), FbsDispatchList.class, params);
              for (FbsDispatchList fbsDispatchListExcel : listFbsDispatchLists) {
                  fbsDispatchListService.save(fbsDispatchListExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFbsDispatchLists.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
