package org.jeecg.modules.fbsDispatchLists.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.fbsDispatchLists.entity.FbsDispatchLists;
import org.jeecg.modules.fbsDispatchLists.service.IFbsDispatchListsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 销售发货退货单主表（表题）
 * @author： jeecg-boot
 * @date：   2020-10-22
 * @version： V1.0
 */
@RestController
@RequestMapping("/fbsDispatchLists/fbsDispatchLists")
@Slf4j
public class FbsDispatchListsController {
	@Autowired
	private IFbsDispatchListsService fbsDispatchListsService;
	
	/**
	  * 分页列表查询
	 * @param fbsDispatchLists
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FbsDispatchLists>> queryPageList(FbsDispatchLists fbsDispatchLists,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FbsDispatchLists>> result = new Result<IPage<FbsDispatchLists>>();
		QueryWrapper<FbsDispatchLists> queryWrapper = QueryGenerator.initQueryWrapper(fbsDispatchLists, req.getParameterMap());
		Page<FbsDispatchLists> page = new Page<FbsDispatchLists>(pageNo, pageSize);
		IPage<FbsDispatchLists> pageList = fbsDispatchListsService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fbsDispatchLists
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FbsDispatchLists> add(@RequestBody FbsDispatchLists fbsDispatchLists) {
		Result<FbsDispatchLists> result = new Result<FbsDispatchLists>();
		try {
			fbsDispatchListsService.save(fbsDispatchLists);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fbsDispatchLists
	 * @return
	 */
//	@PutMapping(value = "/edit")
//	public Result<FbsDispatchLists> edit(@RequestBody FbsDispatchLists fbsDispatchLists) {
//		Result<FbsDispatchLists> result = new Result<FbsDispatchLists>();
//		FbsDispatchLists fbsDispatchListsEntity = fbsDispatchListsService.getById(fbsDispatchLists.getId());
//		if(fbsDispatchListsEntity==null) {
//			result.error500("未找到对应实体");
//		}else {
//			boolean ok = fbsDispatchListsService.updateById(fbsDispatchLists);
//			//TODO 返回false说明什么？
//			if(ok) {
//				result.success("修改成功!");
//			}
//		}
//
//		return result;
//	}
//
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FbsDispatchLists> delete(@RequestParam(name="id",required=true) String id) {
		Result<FbsDispatchLists> result = new Result<FbsDispatchLists>();
		FbsDispatchLists fbsDispatchLists = fbsDispatchListsService.getById(id);
		if(fbsDispatchLists==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsDispatchListsService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FbsDispatchLists> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FbsDispatchLists> result = new Result<FbsDispatchLists>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fbsDispatchListsService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FbsDispatchLists> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FbsDispatchLists> result = new Result<FbsDispatchLists>();
		FbsDispatchLists fbsDispatchLists = fbsDispatchListsService.getById(id);
		if(fbsDispatchLists==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fbsDispatchLists);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FbsDispatchLists> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FbsDispatchLists fbsDispatchLists = JSON.parseObject(deString, FbsDispatchLists.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fbsDispatchLists, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FbsDispatchLists> pageList = fbsDispatchListsService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "销售发货退货单主表（表题）列表");
      mv.addObject(NormalExcelConstants.CLASS, FbsDispatchLists.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("销售发货退货单主表（表题）列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FbsDispatchLists> listFbsDispatchListss = ExcelImportUtil.importExcel(file.getInputStream(), FbsDispatchLists.class, params);
              for (FbsDispatchLists fbsDispatchListsExcel : listFbsDispatchListss) {
                  fbsDispatchListsService.save(fbsDispatchListsExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFbsDispatchListss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 月度项目发货统计
  * @Date 2020/10/22 13:14
  * @Param []
  * @return org.jeecg.common.api.vo.Result<java.util.Map<java.lang.String,java.lang.Object>>
  */
  @GetMapping(value = "/getProjectDelivery")
  public Result<Map<String,Object>> getProjectDelivery(){
	  Result<Map<String,Object>>  result=new Result<>();
	  try {
		  Map<String,Object> map=fbsDispatchListsService.getProjectDelivery();
		  result.setResult(map);
		  result.success("操作成功");
	  } catch (Exception e) {
		  e.printStackTrace();
		  result.error500("操作失败："+e.getMessage());
	  }
	  return result;
  }

}
