package org.jeecg.modules.fbsInventory.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.fbsInventory.entity.FbsInventory;
import org.jeecg.modules.fbsInventory.service.IFbsInventoryService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 存库档案
 * @author： jeecg-boot
 * @date：   2020-10-16
 * @version： V1.0
 */
@RestController
@RequestMapping("/fbsInventory/fbsInventory")
@Slf4j
public class FbsInventoryController {
	@Autowired
	private IFbsInventoryService fbsInventoryService;

	 /**
	  * 安全库存（仓库管理看板）
 	  * @return
	  */
	 @RequestMapping(value = "/getStockList")
	 public Result<List<Map<String,Object>>> getStockList(){
		 Result<List<Map<String,Object>>> result=new Result<>();

		 try {
			 List<Map<String,Object>> map=fbsInventoryService.getStockList();
			 result.setResult(map);
			 result.success("操作成功");

		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }

		 return result;
	 }






	/**
	  * 分页列表查询
	 * @param fbsInventory
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FbsInventory>> queryPageList(FbsInventory fbsInventory,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FbsInventory>> result = new Result<IPage<FbsInventory>>();
		QueryWrapper<FbsInventory> queryWrapper = QueryGenerator.initQueryWrapper(fbsInventory, req.getParameterMap());
		Page<FbsInventory> page = new Page<FbsInventory>(pageNo, pageSize);
		IPage<FbsInventory> pageList = fbsInventoryService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fbsInventory
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FbsInventory> add(@RequestBody FbsInventory fbsInventory) {
		Result<FbsInventory> result = new Result<FbsInventory>();
		try {
			fbsInventoryService.save(fbsInventory);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fbsInventory
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FbsInventory> edit(@RequestBody FbsInventory fbsInventory) {
		Result<FbsInventory> result = new Result<FbsInventory>();
		FbsInventory fbsInventoryEntity = fbsInventoryService.getById(fbsInventory.getId());
		if(fbsInventoryEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsInventoryService.updateById(fbsInventory);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FbsInventory> delete(@RequestParam(name="id",required=true) String id) {
		Result<FbsInventory> result = new Result<FbsInventory>();
		FbsInventory fbsInventory = fbsInventoryService.getById(id);
		if(fbsInventory==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsInventoryService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FbsInventory> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FbsInventory> result = new Result<FbsInventory>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fbsInventoryService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FbsInventory> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FbsInventory> result = new Result<FbsInventory>();
		FbsInventory fbsInventory = fbsInventoryService.getById(id);
		if(fbsInventory==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fbsInventory);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FbsInventory> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FbsInventory fbsInventory = JSON.parseObject(deString, FbsInventory.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fbsInventory, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FbsInventory> pageList = fbsInventoryService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "存库档案列表");
      mv.addObject(NormalExcelConstants.CLASS, FbsInventory.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("存库档案列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FbsInventory> listFbsInventorys = ExcelImportUtil.importExcel(file.getInputStream(), FbsInventory.class, params);
              for (FbsInventory fbsInventoryExcel : listFbsInventorys) {
                  fbsInventoryService.save(fbsInventoryExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFbsInventorys.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
