package org.jeecg.modules.fbsMomOrder.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.fbsMomOrder.entity.FbsMomOrderdetail;
import org.jeecg.modules.fbsMomOrder.service.IFbsMomOrderdetailService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 生产订单子表
 * @author： jeecg-boot
 * @date：   2020-10-16
 * @version： V1.0
 */
@RestController
@RequestMapping("/fbsMomOrder/fbsMomOrderdetail")
@Slf4j
public class FbsMomOrderdetailController {
	@Autowired
	private IFbsMomOrderdetailService fbsMomOrderdetailService;
	
	/**
	  * 分页列表查询
	 * @param fbsMomOrderdetail
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FbsMomOrderdetail>> queryPageList(FbsMomOrderdetail fbsMomOrderdetail,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FbsMomOrderdetail>> result = new Result<IPage<FbsMomOrderdetail>>();
		QueryWrapper<FbsMomOrderdetail> queryWrapper = QueryGenerator.initQueryWrapper(fbsMomOrderdetail, req.getParameterMap());
		Page<FbsMomOrderdetail> page = new Page<FbsMomOrderdetail>(pageNo, pageSize);
		IPage<FbsMomOrderdetail> pageList = fbsMomOrderdetailService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fbsMomOrderdetail
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FbsMomOrderdetail> add(@RequestBody FbsMomOrderdetail fbsMomOrderdetail) {
		Result<FbsMomOrderdetail> result = new Result<FbsMomOrderdetail>();
		try {
			fbsMomOrderdetailService.save(fbsMomOrderdetail);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fbsMomOrderdetail
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FbsMomOrderdetail> edit(@RequestBody FbsMomOrderdetail fbsMomOrderdetail) {
		Result<FbsMomOrderdetail> result = new Result<FbsMomOrderdetail>();
		FbsMomOrderdetail fbsMomOrderdetailEntity = fbsMomOrderdetailService.getById(fbsMomOrderdetail.getId());
		if(fbsMomOrderdetailEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsMomOrderdetailService.updateById(fbsMomOrderdetail);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FbsMomOrderdetail> delete(@RequestParam(name="id",required=true) String id) {
		Result<FbsMomOrderdetail> result = new Result<FbsMomOrderdetail>();
		FbsMomOrderdetail fbsMomOrderdetail = fbsMomOrderdetailService.getById(id);
		if(fbsMomOrderdetail==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsMomOrderdetailService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FbsMomOrderdetail> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FbsMomOrderdetail> result = new Result<FbsMomOrderdetail>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fbsMomOrderdetailService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FbsMomOrderdetail> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FbsMomOrderdetail> result = new Result<FbsMomOrderdetail>();
		FbsMomOrderdetail fbsMomOrderdetail = fbsMomOrderdetailService.getById(id);
		if(fbsMomOrderdetail==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fbsMomOrderdetail);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FbsMomOrderdetail> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FbsMomOrderdetail fbsMomOrderdetail = JSON.parseObject(deString, FbsMomOrderdetail.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fbsMomOrderdetail, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FbsMomOrderdetail> pageList = fbsMomOrderdetailService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "生产订单子表列表");
      mv.addObject(NormalExcelConstants.CLASS, FbsMomOrderdetail.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("生产订单子表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FbsMomOrderdetail> listFbsMomOrderdetails = ExcelImportUtil.importExcel(file.getInputStream(), FbsMomOrderdetail.class, params);
              for (FbsMomOrderdetail fbsMomOrderdetailExcel : listFbsMomOrderdetails) {
                  fbsMomOrderdetailService.save(fbsMomOrderdetailExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFbsMomOrderdetails.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 查询生产计划完成率
  * @Date 2020/10/21 12:49
  * @Param []
  * @return org.jeecg.common.api.vo.Result<java.util.Map<java.lang.String,java.lang.Object>>
  */
  @GetMapping(value = "/getFbsMomOrderdetailSum")
  public Result<Map<String,Object>> getFbsMomOrderdetailSum(){
	  Result<Map<String,Object>> result=new Result<>();
	  try {
		  Map<String,Object> map=fbsMomOrderdetailService.getFbsMomOrderdetailSum();
		  result.setResult(map);
		  result.success("操作成功");
	  } catch (Exception e) {
		  e.printStackTrace();
		  result.error500("操作失败："+e.getMessage());
	  }
	  return result;
  }

  /***
  * @Author chenchuang
  * @Description //TODO 周成品入库数量
  * @Date 2020/10/21 22:03
  * @Param []
  * @return org.jeecg.common.api.vo.Result<java.util.Map<java.lang.String,java.lang.Object>>
  */
  @GetMapping(value = "/getQualifiedInQtyWeek")
  public Result<Map<String,Object>> getQualifiedInQtyWeek(){
	  Result<Map<String,Object>> result=new Result<>();
	  try {
		  Map<String,Object> map=fbsMomOrderdetailService.getQualifiedInQtyWeek();
		  result.setResult(map);
		  result.success("操作成功");
	  } catch (Exception e) {
		  e.printStackTrace();
		  result.error500("操作失败："+e.getMessage());
	  }
	  return result;
  }

}
