package org.jeecg.modules.fbsPerson.controller;

import java.text.SimpleDateFormat;
import java.util.*;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.fbsPerson.entity.FbsPerson;
import org.jeecg.modules.fbsPerson.service.IFbsPersonService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.fbsWorkshopDispatchList.entity.FbsWorkshopDispatchList;
import org.jeecg.modules.fbsWorkshopDispatchList.service.IFbsWorkshopDispatchListService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 人员档案
 * @author： jeecg-boot
 * @date：   2020-10-16
 * @version： V1.0
 */
@RestController
@RequestMapping("/fbsPerson/fbsPerson")
@Slf4j
public class FbsPersonController {
	@Autowired
	private IFbsPersonService fbsPersonService;
	 @Autowired
	 private IFbsWorkshopDispatchListService fbsWorkshopDispatchListService;


	 /**
	  * 仓库管理看板--仓库人员信息数据接口
	  * @return
	  */
	 @RequestMapping(value = "/getWarehousePersonList")
	 public List<FbsPerson> getWarehousePersonList(){
		 List<FbsPerson> list = new ArrayList<>();
		 SimpleDateFormat dateFm = new SimpleDateFormat("HH:mm");
		 List<FbsWorkshopDispatchList> list1 = fbsWorkshopDispatchListService.selNowDispatchList();
		 int i = 1;
		 for (FbsWorkshopDispatchList o:list1){
		 	i++;
			 List<FbsPerson> personList = fbsPersonService.getPerByCode(o.getPersonnelIdU8());
			 if(personList.size() > 0){
			 	if(o.getFirstTime() == null){
					o.setFirstTime(new Date());
				}
				 personList.get(0).setInTime(dateFm.format(o.getFirstTime()));
				 personList.get(0).setRealName(o.getName());
				 if(i%2 == 0){
					 personList.get(0).setAvatar("/images/avatar/avata_2.jpg");
				 }else{
					 personList.get(0).setAvatar("/images/avatar/avata_5.jpg");
				 }

//				 personList.get(0).setInTime(o.getFirstTime());
//				 personList.get(0).setInTime(o.getFirstTime());
//				 personList.get(0).setInTime(o.getFirstTime());
				 list.add(personList.get(0));
			 }
		 }

		 return list;
	 }











	/**
	  * 分页列表查询
	 * @param fbsPerson
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FbsPerson>> queryPageList(FbsPerson fbsPerson,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FbsPerson>> result = new Result<IPage<FbsPerson>>();
		QueryWrapper<FbsPerson> queryWrapper = QueryGenerator.initQueryWrapper(fbsPerson, req.getParameterMap());
		Page<FbsPerson> page = new Page<FbsPerson>(pageNo, pageSize);
		IPage<FbsPerson> pageList = fbsPersonService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fbsPerson
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FbsPerson> add(@RequestBody FbsPerson fbsPerson) {
		Result<FbsPerson> result = new Result<FbsPerson>();
		try {
			fbsPersonService.save(fbsPerson);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fbsPerson
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FbsPerson> edit(@RequestBody FbsPerson fbsPerson) {
		Result<FbsPerson> result = new Result<FbsPerson>();
		FbsPerson fbsPersonEntity = fbsPersonService.getById(fbsPerson.getId());
		if(fbsPersonEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsPersonService.updateById(fbsPerson);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FbsPerson> delete(@RequestParam(name="id",required=true) String id) {
		Result<FbsPerson> result = new Result<FbsPerson>();
		FbsPerson fbsPerson = fbsPersonService.getById(id);
		if(fbsPerson==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fbsPersonService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FbsPerson> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FbsPerson> result = new Result<FbsPerson>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fbsPersonService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FbsPerson> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FbsPerson> result = new Result<FbsPerson>();
		FbsPerson fbsPerson = fbsPersonService.getById(id);
		if(fbsPerson==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fbsPerson);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FbsPerson> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FbsPerson fbsPerson = JSON.parseObject(deString, FbsPerson.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fbsPerson, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FbsPerson> pageList = fbsPersonService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "人员档案列表");
      mv.addObject(NormalExcelConstants.CLASS, FbsPerson.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("人员档案列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FbsPerson> listFbsPersons = ExcelImportUtil.importExcel(file.getInputStream(), FbsPerson.class, params);
              for (FbsPerson fbsPersonExcel : listFbsPersons) {
                  fbsPersonService.save(fbsPersonExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFbsPersons.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
