package org.jeecg.modules.finance.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.finance.entity.FdFinancialStatement;
import org.jeecg.modules.finance.service.IFdFinancialStatementService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 财务报表主表
 * @author： jeecg-boot
 * @date：   2019-07-26
 * @version： V1.0
 */
@RestController
@RequestMapping("/finance/fdFinancialStatement")
@Slf4j
public class FdFinancialStatementController {
	@Autowired
	private IFdFinancialStatementService fdFinancialStatementService;
	
	/**
	  * 分页列表查询
	 * @param fdFinancialStatement
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdFinancialStatement>> queryPageList(FdFinancialStatement fdFinancialStatement,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdFinancialStatement>> result = new Result<IPage<FdFinancialStatement>>();
		QueryWrapper<FdFinancialStatement> queryWrapper = QueryGenerator.initQueryWrapper(fdFinancialStatement, req.getParameterMap());
		Page<FdFinancialStatement> page = new Page<FdFinancialStatement>(pageNo, pageSize);
		IPage<FdFinancialStatement> pageList = fdFinancialStatementService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdFinancialStatement
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdFinancialStatement> add(@RequestBody FdFinancialStatement fdFinancialStatement) {
		Result<FdFinancialStatement> result = new Result<FdFinancialStatement>();
		try {
			fdFinancialStatementService.save(fdFinancialStatement);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdFinancialStatement
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdFinancialStatement> edit(@RequestBody FdFinancialStatement fdFinancialStatement) {
		Result<FdFinancialStatement> result = new Result<FdFinancialStatement>();
		FdFinancialStatement fdFinancialStatementEntity = fdFinancialStatementService.getById(fdFinancialStatement.getId());
		if(fdFinancialStatementEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdFinancialStatementService.updateById(fdFinancialStatement);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdFinancialStatement> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdFinancialStatement> result = new Result<FdFinancialStatement>();
		FdFinancialStatement fdFinancialStatement = fdFinancialStatementService.getById(id);
		if(fdFinancialStatement==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdFinancialStatementService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdFinancialStatement> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdFinancialStatement> result = new Result<FdFinancialStatement>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdFinancialStatementService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdFinancialStatement> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdFinancialStatement> result = new Result<FdFinancialStatement>();
		FdFinancialStatement fdFinancialStatement = fdFinancialStatementService.getById(id);
		if(fdFinancialStatement==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdFinancialStatement);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdFinancialStatement> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdFinancialStatement fdFinancialStatement = JSON.parseObject(deString, FdFinancialStatement.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdFinancialStatement, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdFinancialStatement> pageList = fdFinancialStatementService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "财务报表主表列表");
      mv.addObject(NormalExcelConstants.CLASS, FdFinancialStatement.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("财务报表主表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdFinancialStatement> listFdFinancialStatements = ExcelImportUtil.importExcel(file.getInputStream(), FdFinancialStatement.class, params);
              for (FdFinancialStatement fdFinancialStatementExcel : listFdFinancialStatements) {
                  fdFinancialStatementService.save(fdFinancialStatementExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdFinancialStatements.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
