package org.jeecg.modules.finance.controller;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdPersonnel;
import org.jeecg.modules.basedata.service.IFdPersonnelService;
import org.jeecg.modules.finance.entity.FdFinancialStatements;
import org.jeecg.modules.finance.service.IFdFinancialStatementsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.projectManagement.entity.FdProjectFileCompound;
import org.jeecg.modules.projectManagement.entity.FdProjectQuotation;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 财务报表
 * @author： jeecg-boot
 * @date： 2019-07-24
 * @version： V1.0
 */
@RestController
@RequestMapping("/finance/fdFinancialStatements")
@Slf4j
public class FdFinancialStatementsController {
    @Autowired
    private IFdFinancialStatementsService fdFinancialStatementsService;
    @Resource
    private IFdPersonnelService fdPersonnelService;

    /**
     * 分页列表查询
     *
     * @param fdFinancialStatements
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdFinancialStatements>> queryPageList(FdFinancialStatements fdFinancialStatements,
                                                              @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                              @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                              HttpServletRequest req) {
        Result<IPage<FdFinancialStatements>> result = new Result<IPage<FdFinancialStatements>>();
        QueryWrapper<FdFinancialStatements> queryWrapper = QueryGenerator.initQueryWrapper(fdFinancialStatements, req.getParameterMap());
        Page<FdFinancialStatements> page = new Page<FdFinancialStatements>(pageNo, pageSize);
        IPage<FdFinancialStatements> pageList = fdFinancialStatementsService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdFinancialStatements
     * @return
     */
    @PostMapping(value = "/add")
    public Result<FdFinancialStatements> add(@RequestBody FdFinancialStatements fdFinancialStatements) {
        Result<FdFinancialStatements> result = new Result<FdFinancialStatements>();
        try {
            fdFinancialStatementsService.save(fdFinancialStatements);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdFinancialStatements
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<FdFinancialStatements> edit(@RequestBody FdFinancialStatements fdFinancialStatements) {
        Result<FdFinancialStatements> result = new Result<FdFinancialStatements>();
        FdFinancialStatements fdFinancialStatementsEntity = fdFinancialStatementsService.getById(fdFinancialStatements.getId());
        if (fdFinancialStatementsEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdFinancialStatementsService.updateById(fdFinancialStatements);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<FdFinancialStatements> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdFinancialStatements> result = new Result<FdFinancialStatements>();
        FdFinancialStatements fdFinancialStatements = fdFinancialStatementsService.getById(id);
        if (fdFinancialStatements == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdFinancialStatementsService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdFinancialStatements> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdFinancialStatements> result = new Result<FdFinancialStatements>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdFinancialStatementsService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdFinancialStatements> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdFinancialStatements> result = new Result<FdFinancialStatements>();
        FdFinancialStatements fdFinancialStatements = fdFinancialStatementsService.getById(id);
        if (fdFinancialStatements == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdFinancialStatements);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdFinancialStatements> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdFinancialStatements fdFinancialStatements = JSON.parseObject(deString, FdFinancialStatements.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdFinancialStatements, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdFinancialStatements> pageList = fdFinancialStatementsService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "财务报表列表");
        mv.addObject(NormalExcelConstants.CLASS, FdFinancialStatements.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("财务报表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdFinancialStatements> listFdFinancialStatementss = ExcelImportUtil.importExcel(file.getInputStream(), FdFinancialStatements.class, params);
                for (FdFinancialStatements fdFinancialStatementsExcel : listFdFinancialStatementss) {
                    fdFinancialStatementsService.save(fdFinancialStatementsExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdFinancialStatementss.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.finance.entity.FdFinancialStatements>
     * @Author gurui
     * @Description //TODO 查询报表名称为本月到款情况
     * @Date 15:40 2019/7/24
     * @Param [id]
     **/
    @PostMapping(value = "/queryByConId1")
    @RequiresPermissions("url:statement:view")
    public Result<List<FdFinancialStatements>> queryByConId1(@RequestBody FdFinancialStatements financialStatements) {
        Result<List<FdFinancialStatements>> result = new Result<>();
        List<FdFinancialStatements> fdFinancialStatements1 = fdFinancialStatementsService.queryByConId1(financialStatements);
        if (fdFinancialStatements1 == null) {
            result.error500("未找到对应实体");
        } else {
            // for (FdFinancialStatements fdFinancialStatements:fdFinancialStatements1){
            //     //商务经理名称
            //     LambdaQueryWrapper<FdPersonnel> queryWrapper = new LambdaQueryWrapper<>();
            //     queryWrapper.eq(FdPersonnel::getId, fdFinancialStatements.getBdId());
            //     FdPersonnel bd = fdPersonnelService.getOne(queryWrapper);
            //     fdFinancialStatements.setBdName(bd.getName());
            //     fdFinancialStatements.setPkReportName("2");
            //     fdFinancialStatements.setPkReportName("已发付款通知书,但未收到款的合同");
            //     //开始结束时间查询
            //     QueryWrapper<FdFinancialStatements> queryWrapper1 = new QueryWrapper<>();
            //     if (fdFinancialStatements.getStartDate()!=null&&fdFinancialStatements.getEndDate()!=null){
            //         queryWrapper1.between("prb.payment_date",fdFinancialStatements.getStartDate(),fdFinancialStatements.getEndDate());
            //     }
            // }
            result.setResult(fdFinancialStatements1);
            result.setSuccess(true);
        }
        return result;
    }
    /**
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.finance.entity.FdFinancialStatements>
     * @Author gurui
     * @Description //TODO 查询已发付款通知书,但未收到款的合同
     * @Date 15:40 2019/7/24
     * @Param [id]
     **/
    @PostMapping(value = "/queryByConId2")
    @RequiresPermissions("url:statement:view")
    public Result<List<FdFinancialStatements>> queryByConId2(@RequestBody FdFinancialStatements financialStatements) {
        Result<List<FdFinancialStatements>> result = new Result<>();
        List<FdFinancialStatements> fdFinancialStatements1 = fdFinancialStatementsService.queryByConId2(financialStatements);
        if (fdFinancialStatements1 == null) {
            result.error500("未找到对应实体");
        } else {
            // for (FdFinancialStatements fdFinancialStatements:fdFinancialStatements1){
            //     //商务经理名称
            //     LambdaQueryWrapper<FdPersonnel> queryWrapper = new LambdaQueryWrapper<>();
            //     queryWrapper.eq(FdPersonnel::getId, fdFinancialStatements.getBdId());
            //     FdPersonnel bd = fdPersonnelService.getOne(queryWrapper);
            //     fdFinancialStatements.setBdName(bd.getName());
            //     fdFinancialStatements.setPkReportName("2");
            //     fdFinancialStatements.setPkReportName("已发付款通知书,但未收到款的合同");
            //     //开始结束时间查询
            //     QueryWrapper<FdFinancialStatements> queryWrapper1 = new QueryWrapper<>();
            //     if (fdFinancialStatements.getStartDate()!=null&&fdFinancialStatements.getEndDate()!=null){
            //         queryWrapper1.between("prb.notice_date",fdFinancialStatements.getStartDate(),fdFinancialStatements.getEndDate());
            //     }
            // }
            result.setResult(fdFinancialStatements1);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.finance.entity.FdFinancialStatements>
     * @Author gurui
     * @Description //TODO 导出json
     * @Date 17:01 2019/7/24
     * @Param [id]
     **/
    @PostMapping(value = "/export")
    public Result<FdFinancialStatements> export(@RequestBody String id) {
        Result<FdFinancialStatements> result = new Result<>();
        try {
            FdFinancialStatements fdFinancialStatements = fdFinancialStatementsService.export(id);
            result.setResult(fdFinancialStatements);
            result.success("导出成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("导出失败" + e.getMessage());
        }
        return result;
    }


}
