package org.jeecg.modules.finance.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.finance.entity.FdSupplementaryContractReceivablesB;
import org.jeecg.modules.finance.service.IFdSupplementaryContractReceivablesBService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 补充合同应收款子表
 * @author： gur
 * @date：   2019-07-19
 * @version： V1.0
 */
@RestController
@RequestMapping("/finance/fdSupplementaryContractReceivablesB")
@Slf4j
public class FdSupplementaryContractReceivablesBController {
	@Autowired
	private IFdSupplementaryContractReceivablesBService fdSupplementaryContractReceivablesBService;
	
	/**
	  * 分页列表查询
	 * @param fdSupplementaryContractReceivablesB
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdSupplementaryContractReceivablesB>> queryPageList(FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesB,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdSupplementaryContractReceivablesB>> result = new Result<IPage<FdSupplementaryContractReceivablesB>>();
		QueryWrapper<FdSupplementaryContractReceivablesB> queryWrapper = QueryGenerator.initQueryWrapper(fdSupplementaryContractReceivablesB, req.getParameterMap());
		Page<FdSupplementaryContractReceivablesB> page = new Page<FdSupplementaryContractReceivablesB>(pageNo, pageSize);
		IPage<FdSupplementaryContractReceivablesB> pageList = fdSupplementaryContractReceivablesBService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdSupplementaryContractReceivablesB
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdSupplementaryContractReceivablesB> add(@RequestBody FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesB) {
		Result<FdSupplementaryContractReceivablesB> result = new Result<FdSupplementaryContractReceivablesB>();
		try {
			fdSupplementaryContractReceivablesBService.save(fdSupplementaryContractReceivablesB);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdSupplementaryContractReceivablesB
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdSupplementaryContractReceivablesB> edit(@RequestBody FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesB) {
		Result<FdSupplementaryContractReceivablesB> result = new Result<FdSupplementaryContractReceivablesB>();
		FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesBEntity = fdSupplementaryContractReceivablesBService.getById(fdSupplementaryContractReceivablesB.getId());
		if(fdSupplementaryContractReceivablesBEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdSupplementaryContractReceivablesBService.updateById(fdSupplementaryContractReceivablesB);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdSupplementaryContractReceivablesB> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdSupplementaryContractReceivablesB> result = new Result<FdSupplementaryContractReceivablesB>();
		FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesB = fdSupplementaryContractReceivablesBService.getById(id);
		if(fdSupplementaryContractReceivablesB==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdSupplementaryContractReceivablesBService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdSupplementaryContractReceivablesB> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdSupplementaryContractReceivablesB> result = new Result<FdSupplementaryContractReceivablesB>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdSupplementaryContractReceivablesBService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdSupplementaryContractReceivablesB> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdSupplementaryContractReceivablesB> result = new Result<FdSupplementaryContractReceivablesB>();
		FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesB = fdSupplementaryContractReceivablesBService.getById(id);
		if(fdSupplementaryContractReceivablesB==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdSupplementaryContractReceivablesB);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdSupplementaryContractReceivablesB> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesB = JSON.parseObject(deString, FdSupplementaryContractReceivablesB.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdSupplementaryContractReceivablesB, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdSupplementaryContractReceivablesB> pageList = fdSupplementaryContractReceivablesBService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "补充合同应收款子表列表");
      mv.addObject(NormalExcelConstants.CLASS, FdSupplementaryContractReceivablesB.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("补充合同应收款子表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdSupplementaryContractReceivablesB> listFdSupplementaryContractReceivablesBs = ExcelImportUtil.importExcel(file.getInputStream(), FdSupplementaryContractReceivablesB.class, params);
              for (FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesBExcel : listFdSupplementaryContractReceivablesBs) {
                  fdSupplementaryContractReceivablesBService.save(fdSupplementaryContractReceivablesBExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdSupplementaryContractReceivablesBs.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }


	 /**
	  * @Author gurui
	  * @Description //TODO 根据合同id查询相关信息
	  * @Date 14:18 2019/7/19
	  * @Param [contractId]
	  * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.finance.entity.FdSupplementaryContractReceivablesB>
	  **/
	 @GetMapping(value = "/queryByConId")
	 public Result<FdSupplementaryContractReceivablesB> queryByConId(@RequestParam(name = "contractId", required = true) String contractId) {
		 Result<FdSupplementaryContractReceivablesB> result = new Result<>();
		 FdSupplementaryContractReceivablesB fdSupplementaryContractReceivablesb = fdSupplementaryContractReceivablesBService.queryByConId(contractId);
		 if (fdSupplementaryContractReceivablesb == null) {
			 result.error500("未找到对应实体");
		 } else {
			 String dict = fdSupplementaryContractReceivablesBService.queryByDict(fdSupplementaryContractReceivablesb.getPkMilestone());
			 fdSupplementaryContractReceivablesb.setMilestoneName(dict);
			 result.setResult(fdSupplementaryContractReceivablesb);
			 result.setSuccess(true);
		 }
		 return result;
	 }

}
