package org.jeecg.modules.financeManage.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.financeManage.entity.FdClinicalIncomeBudgetCost;
import org.jeecg.modules.financeManage.service.IFdClinicalIncomeBudgetCostService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 临床预算成本
 * @author： jeecg-boot
 * @date：   2020-06-23
 * @version： V1.0
 */
@RestController
@RequestMapping("/financeManage/fdClinicalIncomeBudgetCost")
@Slf4j
public class FdClinicalIncomeBudgetCostController {
	@Autowired
	private IFdClinicalIncomeBudgetCostService fdClinicalIncomeBudgetCostService;
	
	/**
	  * 分页列表查询
	 * @param fdClinicalIncomeBudgetCost
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdClinicalIncomeBudgetCost>> queryPageList(FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCost,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdClinicalIncomeBudgetCost>> result = new Result<IPage<FdClinicalIncomeBudgetCost>>();
		QueryWrapper<FdClinicalIncomeBudgetCost> queryWrapper = QueryGenerator.initQueryWrapper(fdClinicalIncomeBudgetCost, req.getParameterMap());
		queryWrapper.lambda().eq(FdClinicalIncomeBudgetCost::getDelFlag,"0");
		queryWrapper.lambda().orderByAsc(FdClinicalIncomeBudgetCost::getSort);
		Page<FdClinicalIncomeBudgetCost> page = new Page<FdClinicalIncomeBudgetCost>(pageNo, pageSize);
		IPage<FdClinicalIncomeBudgetCost> pageList = fdClinicalIncomeBudgetCostService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdClinicalIncomeBudgetCost
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdClinicalIncomeBudgetCost> add(@RequestBody FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCost) {
		Result<FdClinicalIncomeBudgetCost> result = new Result<FdClinicalIncomeBudgetCost>();
		try {
			fdClinicalIncomeBudgetCostService.save(fdClinicalIncomeBudgetCost);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdClinicalIncomeBudgetCost
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdClinicalIncomeBudgetCost> edit(@RequestBody FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCost) {
		Result<FdClinicalIncomeBudgetCost> result = new Result<FdClinicalIncomeBudgetCost>();
		FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCostEntity = fdClinicalIncomeBudgetCostService.getById(fdClinicalIncomeBudgetCost.getId());
		if(fdClinicalIncomeBudgetCostEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdClinicalIncomeBudgetCostService.updateById(fdClinicalIncomeBudgetCost);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdClinicalIncomeBudgetCost> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdClinicalIncomeBudgetCost> result = new Result<FdClinicalIncomeBudgetCost>();
		FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCost = fdClinicalIncomeBudgetCostService.getById(id);
		if(fdClinicalIncomeBudgetCost==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdClinicalIncomeBudgetCostService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdClinicalIncomeBudgetCost> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdClinicalIncomeBudgetCost> result = new Result<FdClinicalIncomeBudgetCost>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdClinicalIncomeBudgetCostService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdClinicalIncomeBudgetCost> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdClinicalIncomeBudgetCost> result = new Result<FdClinicalIncomeBudgetCost>();
		FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCost = fdClinicalIncomeBudgetCostService.getById(id);
		if(fdClinicalIncomeBudgetCost==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdClinicalIncomeBudgetCost);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdClinicalIncomeBudgetCost> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCost = JSON.parseObject(deString, FdClinicalIncomeBudgetCost.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdClinicalIncomeBudgetCost, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdClinicalIncomeBudgetCost> pageList = fdClinicalIncomeBudgetCostService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "临床收入确认列表");
      mv.addObject(NormalExcelConstants.CLASS, FdClinicalIncomeBudgetCost.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("临床收入确认列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdClinicalIncomeBudgetCost> listFdClinicalIncomeBudgetCosts = ExcelImportUtil.importExcel(file.getInputStream(), FdClinicalIncomeBudgetCost.class, params);
              for (FdClinicalIncomeBudgetCost fdClinicalIncomeBudgetCostExcel : listFdClinicalIncomeBudgetCosts) {
                  fdClinicalIncomeBudgetCostService.save(fdClinicalIncomeBudgetCostExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdClinicalIncomeBudgetCosts.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
