package org.jeecg.modules.financeManage.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.extension.api.R;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.financeManage.entity.FdClinicalIncome;
import org.jeecg.modules.financeManage.service.IFdClinicalIncomeService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.projectManagement.entity.FdContract;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 临床收入确认
 * @author： jeecg-boot
 * @date：   2020-06-23
 * @version： V1.0
 */
@RestController
@RequestMapping("/financeManage/fdClinicalIncome")
@Slf4j
public class FdClinicalIncomeController {
	@Autowired
	private IFdClinicalIncomeService fdClinicalIncomeService;
	
	/**
	  * 分页列表查询
	 * @param fdClinicalIncome
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdClinicalIncome>> queryPageList(FdClinicalIncome fdClinicalIncome,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdClinicalIncome>> result = new Result<IPage<FdClinicalIncome>>();
		QueryWrapper<FdClinicalIncome> queryWrapper = QueryGenerator.initQueryWrapper(fdClinicalIncome, req.getParameterMap());
		queryWrapper.lambda().eq(FdClinicalIncome::getDelFlag,"0");
		queryWrapper.lambda().orderByDesc(FdClinicalIncome::getCreateTime);
		Page<FdClinicalIncome> page = new Page<FdClinicalIncome>(pageNo, pageSize);
		IPage<FdClinicalIncome> pageList = fdClinicalIncomeService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdClinicalIncome
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdClinicalIncome> add(@RequestBody FdClinicalIncome fdClinicalIncome) {
		Result<FdClinicalIncome> result = new Result<FdClinicalIncome>();
		try {
			fdClinicalIncomeService.save(fdClinicalIncome);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdClinicalIncome
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdClinicalIncome> edit(@RequestBody FdClinicalIncome fdClinicalIncome) {
		Result<FdClinicalIncome> result = new Result<FdClinicalIncome>();
		FdClinicalIncome fdClinicalIncomeEntity = fdClinicalIncomeService.getById(fdClinicalIncome.getId());
		if(fdClinicalIncomeEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdClinicalIncomeService.updateById(fdClinicalIncome);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdClinicalIncome> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdClinicalIncome> result = new Result<FdClinicalIncome>();
		FdClinicalIncome fdClinicalIncome = fdClinicalIncomeService.getById(id);
		if(fdClinicalIncome==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdClinicalIncomeService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdClinicalIncome> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdClinicalIncome> result = new Result<FdClinicalIncome>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdClinicalIncomeService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdClinicalIncome> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdClinicalIncome> result = new Result<FdClinicalIncome>();
		FdClinicalIncome fdClinicalIncome = fdClinicalIncomeService.getById(id);
		if(fdClinicalIncome==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdClinicalIncome);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdClinicalIncome> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdClinicalIncome fdClinicalIncome = JSON.parseObject(deString, FdClinicalIncome.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdClinicalIncome, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdClinicalIncome> pageList = fdClinicalIncomeService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "临床收入确认列表");
      mv.addObject(NormalExcelConstants.CLASS, FdClinicalIncome.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("临床收入确认列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdClinicalIncome> listFdClinicalIncomes = ExcelImportUtil.importExcel(file.getInputStream(), FdClinicalIncome.class, params);
              for (FdClinicalIncome fdClinicalIncomeExcel : listFdClinicalIncomes) {
                  fdClinicalIncomeService.save(fdClinicalIncomeExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdClinicalIncomes.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 新增主表以及主表数据
  * @Date 2020/6/28 15:52
  * @Param [fdClinicalIncome]
  * @return org.jeecg.common.api.vo.Result
  */
	 @PostMapping("/saveAll")
	 public Result saveAll(@RequestBody FdClinicalIncome fdClinicalIncome){
		 Result result=new Result();
		 try {
			 fdClinicalIncomeService.saveAll(fdClinicalIncome);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 修改主表以及主表数据
	 * @Date 2020/7/6 16:35
	 * @Param [fdClinicalIncome]
	 * @return org.jeecg.common.api.vo.Result
	 */
	 @PutMapping("/updateAll")
	 public Result updateAll(@RequestBody FdClinicalIncome fdClinicalIncome){
		 Result result=new Result();
		 try {
			 fdClinicalIncomeService.updateAll(fdClinicalIncome);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 根据项目查询相关信息
	 * @Date 2020/6/30 11:14
	 * @Param [projectFileId]
	 * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.financeManage.entity.FdClinicalIncome>
	 */
	 @GetMapping(value = "/getByProjectFileId")
	 public Result<FdClinicalIncome> getByProjectFileId(String projectFileId){
		 Result<FdClinicalIncome> result=new Result<>();
		 try {
			 FdClinicalIncome fdClinicalIncome=fdClinicalIncomeService.getByProjectFileId(projectFileId);
			 result.setResult(fdClinicalIncome);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 根据临床预算新增的项目查询合同号集合
	 * @Date 2020/6/28 17:43
	 * @Param []
	 * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.projectManagement.entity.FdContract>>
	 */
	 @GetMapping(value = "/getContractList")
	 public Result<List<FdContract>> getContractList(){
		 Result<List<FdContract>> result=new Result<>();
		 try {
			 List<FdContract> fdContractList=fdClinicalIncomeService.getContractList();
			 result.setResult(fdContractList);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 根据结算日，刷新项目的进度
	 * @Date 2020/7/10 15:11
	 * @Param []
	 * @return org.jeecg.common.api.vo.Result
	 */
	 @GetMapping(value = "/refreshProgress")
	public Result refreshProgress(){
	 	Result result=new Result();
		try {
			fdClinicalIncomeService.refreshProgress();
			result.success("操作成功");
		} catch (Exception e) {
			e.printStackTrace();
			result.error500("操作失败："+e.getMessage());
		}
		return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 导入相关费用
	* @Date 2020/7/7 15:50
	* @Param [fdClinicalIncome]
	* @return org.jeecg.common.api.vo.Result
	*/
	 @PostMapping(value = "/export")
	public Result export(@RequestBody FdClinicalIncome fdClinicalIncome){
		Result result=new Result();
		 try {
			 fdClinicalIncomeService.export(fdClinicalIncome);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	}

	/**
	* @Author chenchuang
	* @Description //TODO 删除主表和子表
	* @Date 2020/7/7 15:54
	* @Param [id]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.financeManage.entity.FdClinicalIncome>
	*/
	 @DeleteMapping(value = "/deleteAll")
	 public Result<FdClinicalIncome> deleteAll(@RequestParam(name="id",required=true) String id) {
		 Result<FdClinicalIncome> result = new Result<FdClinicalIncome>();
		 FdClinicalIncome fdClinicalIncome = fdClinicalIncomeService.getById(id);
		 if(fdClinicalIncome==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 fdClinicalIncomeService.deleteAll(fdClinicalIncome);
				 result.success("删除成功");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("操作失败："+e.getMessage());
			 }
		 }

		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 导出未结算的主表
	 * @Date 2020/8/7 15:07
	 * @Param []
	 * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.financeManage.entity.FdClinicalIncome>>
	 */
	 @GetMapping(value = "/exportMain")
	 public Result<List<FdClinicalIncome>> exportMain(){
		 Result<List<FdClinicalIncome>> result=new Result<>();
		 try {
			 List<FdClinicalIncome> fdClinicalIncomeList=fdClinicalIncomeService.exportMain();
			 result.setResult(fdClinicalIncomeList);
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

 }
