package org.jeecg.modules.financeManage.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.financeManage.entity.FdIncomeItem;
import org.jeecg.modules.financeManage.service.IFdIncomeItemService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 收入确认明细
 * @author： jeecg-boot
 * @date：   2019-12-02
 * @version： V1.0
 */
@RestController
@RequestMapping("/financeManage/fdIncomeItem")
@Slf4j
public class FdIncomeItemController {
	@Autowired
	private IFdIncomeItemService fdIncomeItemService;
	
	/**
	  * 分页列表查询
	 * @param fdIncomeItem
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdIncomeItem>> queryPageList(FdIncomeItem fdIncomeItem,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdIncomeItem>> result = new Result<IPage<FdIncomeItem>>();
		QueryWrapper<FdIncomeItem> queryWrapper = QueryGenerator.initQueryWrapper(fdIncomeItem, req.getParameterMap());
		Page<FdIncomeItem> page = new Page<FdIncomeItem>(pageNo, pageSize);
		queryWrapper.lambda().orderByAsc(FdIncomeItem::getSort);
		queryWrapper.lambda().eq(FdIncomeItem::getDelFlag, CommonConstant.DEL_FLAG_0.toString());
		IPage<FdIncomeItem> pageList = fdIncomeItemService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdIncomeItem
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdIncomeItem> add(@RequestBody FdIncomeItem fdIncomeItem) {
		Result<FdIncomeItem> result = new Result<FdIncomeItem>();
		try {
			fdIncomeItemService.save(fdIncomeItem);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdIncomeItem
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdIncomeItem> edit(@RequestBody FdIncomeItem fdIncomeItem) {
		Result<FdIncomeItem> result = new Result<FdIncomeItem>();
		FdIncomeItem fdIncomeItemEntity = fdIncomeItemService.getById(fdIncomeItem.getId());
		if(fdIncomeItemEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdIncomeItemService.updateById(fdIncomeItem);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdIncomeItem> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdIncomeItem> result = new Result<FdIncomeItem>();
		FdIncomeItem fdIncomeItem = fdIncomeItemService.getById(id);
		if(fdIncomeItem==null) {
			result.error500("未找到对应实体");
		}else {
			fdIncomeItem.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
			boolean ok = fdIncomeItemService.updateById(fdIncomeItem);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdIncomeItem> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdIncomeItem> result = new Result<FdIncomeItem>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdIncomeItemService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdIncomeItem> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdIncomeItem> result = new Result<FdIncomeItem>();
		FdIncomeItem fdIncomeItem = fdIncomeItemService.getById(id);
		if(fdIncomeItem==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdIncomeItem);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdIncomeItem> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdIncomeItem fdIncomeItem = JSON.parseObject(deString, FdIncomeItem.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdIncomeItem, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdIncomeItem> pageList = fdIncomeItemService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "收入确认明细列表");
      mv.addObject(NormalExcelConstants.CLASS, FdIncomeItem.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("收入确认明细列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdIncomeItem> listFdIncomeItems = ExcelImportUtil.importExcel(file.getInputStream(), FdIncomeItem.class, params);
              for (FdIncomeItem fdIncomeItemExcel : listFdIncomeItems) {
                  fdIncomeItemService.save(fdIncomeItemExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdIncomeItems.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 导出未结算的所有明细数据
  * @Date 2020/8/7 11:42
  * @Param []
  * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.financeManage.entity.FdIncomeItem>>
  */
  @GetMapping(value = "/exportAllItemBySettlementStatus")
  public Result<List<FdIncomeItem>> exportAllItemBySettlementStatus(){
	  Result<List<FdIncomeItem>> result=new Result<>();
	  try {
		  List<FdIncomeItem> fdIncomeItemList=fdIncomeItemService.exportAllItemBySettlementStatus();
		  result.setResult(fdIncomeItemList);
		  result.success("操作成功");
	  } catch (Exception e) {
		  e.printStackTrace();
		  result.error500("操作失败："+e.getMessage());
	  }
	  return result;
  }

}
