package org.jeecg.modules.financeManage.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.AutoLog;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.financeManage.entity.FdSampleStorageRevenueRecognition;
import org.jeecg.modules.financeManage.service.IFdSampleStorageRevenueRecognitionService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 样本储存收入确认
 * @author： jeecg-boot
 * @date：   2020-08-10
 * @version： V1.0
 */
@RestController
@RequestMapping("/financeManage/fdSampleStorageRevenueRecognition")
@Slf4j
public class FdSampleStorageRevenueRecognitionController {
	@Autowired
	private IFdSampleStorageRevenueRecognitionService fdSampleStorageRevenueRecognitionService;
	
	/**
	  * 分页列表查询
	 * @param fdSampleStorageRevenueRecognition
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdSampleStorageRevenueRecognition>> queryPageList(FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdSampleStorageRevenueRecognition>> result = new Result<IPage<FdSampleStorageRevenueRecognition>>();
		//合同号
		String code=fdSampleStorageRevenueRecognition.getCode();
		if(StringUtils.isNotBlank(code)){
			fdSampleStorageRevenueRecognition.setCode(null);
		}
		//客户名称
		String customerName=fdSampleStorageRevenueRecognition.getCustomerName();
		if(StringUtils.isNotBlank(customerName)){
			fdSampleStorageRevenueRecognition.setCustomerName(null);
		}
		//结算状态
		String settlementStatusId=fdSampleStorageRevenueRecognition.getSettlementStatusId();
		if(settlementStatusId!=null&&settlementStatusId.equals("null")){
			fdSampleStorageRevenueRecognition.setSettlementStatusId(null);
		}

		QueryWrapper<FdSampleStorageRevenueRecognition> queryWrapper = QueryGenerator.initQueryWrapper(fdSampleStorageRevenueRecognition, req.getParameterMap());
		Page<FdSampleStorageRevenueRecognition> page = new Page<FdSampleStorageRevenueRecognition>(pageNo, pageSize);
		if(StringUtils.isNotBlank(code)){
			queryWrapper.lambda().like(FdSampleStorageRevenueRecognition::getCode,code);
		}
		if(StringUtils.isNotBlank(customerName)){
			queryWrapper.lambda().like(FdSampleStorageRevenueRecognition::getCustomerName,customerName);
		}
		queryWrapper.lambda().eq(FdSampleStorageRevenueRecognition::getDelFlag,"0");
		queryWrapper.lambda().orderByDesc(FdSampleStorageRevenueRecognition::getCreateTime);
		IPage<FdSampleStorageRevenueRecognition> pageList = fdSampleStorageRevenueRecognitionService.page(page, queryWrapper);

		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdSampleStorageRevenueRecognition
	 * @return
	 */
	@AutoLog(value = "样本存储新增")
	@PostMapping(value = "/add")
	public Result<FdSampleStorageRevenueRecognition> add(@RequestBody FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition) {
		Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		try {
			fdSampleStorageRevenueRecognitionService.save(fdSampleStorageRevenueRecognition);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdSampleStorageRevenueRecognition
	 * @return
	 */
	@AutoLog(value = "样本存储修改")
	@PutMapping(value = "/edit")
	public Result<FdSampleStorageRevenueRecognition> edit(@RequestBody FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition) {
		Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognitionEntity = fdSampleStorageRevenueRecognitionService.getById(fdSampleStorageRevenueRecognition.getId());
		if(fdSampleStorageRevenueRecognitionEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdSampleStorageRevenueRecognitionService.updateById(fdSampleStorageRevenueRecognition);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@AutoLog(value = "样本存储删除")
	@DeleteMapping(value = "/delete")
	public Result<FdSampleStorageRevenueRecognition> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition = fdSampleStorageRevenueRecognitionService.getById(id);
		if(fdSampleStorageRevenueRecognition==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdSampleStorageRevenueRecognitionService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */

	@AutoLog(value = "样本存储批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdSampleStorageRevenueRecognition> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdSampleStorageRevenueRecognitionService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdSampleStorageRevenueRecognition> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition = fdSampleStorageRevenueRecognitionService.getById(id);
		if(fdSampleStorageRevenueRecognition==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdSampleStorageRevenueRecognition);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdSampleStorageRevenueRecognition> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition = JSON.parseObject(deString, FdSampleStorageRevenueRecognition.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdSampleStorageRevenueRecognition, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdSampleStorageRevenueRecognition> pageList = fdSampleStorageRevenueRecognitionService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "样本储存收入确认列表");
      mv.addObject(NormalExcelConstants.CLASS, FdSampleStorageRevenueRecognition.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("样本储存收入确认列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdSampleStorageRevenueRecognition> listFdSampleStorageRevenueRecognitions = ExcelImportUtil.importExcel(file.getInputStream(), FdSampleStorageRevenueRecognition.class, params);
              for (FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognitionExcel : listFdSampleStorageRevenueRecognitions) {
                  fdSampleStorageRevenueRecognitionService.save(fdSampleStorageRevenueRecognitionExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdSampleStorageRevenueRecognitions.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 根据项目id查询相关数据
  * @Date 2020/8/13 14:36
  * @Param [projectId]
  * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.financeManage.entity.FdSampleStorageRevenueRecognition>
  */
  @GetMapping(value = "/getFdSampleStorageRevenueRecognitionByProjectId")
  public Result<FdSampleStorageRevenueRecognition> getFdSampleStorageRevenueRecognitionByProjectId(String projectId){
	  Result<FdSampleStorageRevenueRecognition> result=new Result<>();
	  try {
		  FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition=fdSampleStorageRevenueRecognitionService.getFdSampleStorageRevenueRecognitionByProjectId(projectId);
		  result.setResult(fdSampleStorageRevenueRecognition);
		  result.success("操作成功");
	   } catch (Exception e) {
		  e.printStackTrace();
		  result.error500("操作失败："+e.getMessage());
	  }
	  return result;
  }

	/**
	* @Author chenchuang
	* @Description //TODO 保存
	* @Date 2020/8/13 15:05
	* @Param [fdSampleStorageRevenueRecognition]
	* @return org.jeecg.common.api.vo.Result<org.jeecg.modules.financeManage.entity.FdSampleStorageRevenueRecognition>
	*/
	@AutoLog(value = "样本存储新增")
	 @PostMapping(value = "/addCustom")
	 public Result<FdSampleStorageRevenueRecognition> addCustom(@RequestBody FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition) {
		 Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		 try {
			 fdSampleStorageRevenueRecognitionService.addCustom(fdSampleStorageRevenueRecognition);
			 result.success("添加成功！");
		 } catch (Exception e) {
			 e.printStackTrace();
			 log.info(e.getMessage());
			 result.error500("操作失败");
		 }
		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 修改保存
	 * @Date 2020/8/13 16:13
	 * @Param [fdSampleStorageRevenueRecognition]
	 * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.financeManage.entity.FdSampleStorageRevenueRecognition>
	 */
	 @AutoLog(value = "样本存储修改")
	 @PutMapping(value = "/updateCustom")
	 public Result<FdSampleStorageRevenueRecognition> updateCustom(@RequestBody FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognition) {
		 Result<FdSampleStorageRevenueRecognition> result = new Result<FdSampleStorageRevenueRecognition>();
		 FdSampleStorageRevenueRecognition fdSampleStorageRevenueRecognitionEntity = fdSampleStorageRevenueRecognitionService.getById(fdSampleStorageRevenueRecognition.getId());
		 if(fdSampleStorageRevenueRecognitionEntity==null) {
			 result.error500("未找到对应实体");
		 }else {
			 try {
				 fdSampleStorageRevenueRecognitionService.updateCustom(fdSampleStorageRevenueRecognition);
				 result.success("修改成功");
			 } catch (Exception e) {
				 e.printStackTrace();
				 result.error500("操作失败："+e.getMessage());
			 }
			 //TODO 返回false说明什么？
//			 if(ok) {
//				 result.success("修改成功!");
//			 }
		 }

		 return result;
	 }

	 /**
	 * @Author chenchuang
	 * @Description //TODO 月度更新
	 * @Date 2020/8/19 9:50
	 * @Param []
	 * @return org.jeecg.common.api.vo.Result
	 */
	 @AutoLog(value = "样本存储更新")
	 @GetMapping(value = "/monthlyUpdate")
	 public Result monthlyUpdate(){
	 	Result result =new Result();
		 try {
			 fdSampleStorageRevenueRecognitionService.monthlyUpdate();
			 result.success("操作成功");
		 } catch (Exception e) {
			 e.printStackTrace();
			 result.error500("操作失败："+e.getMessage());
		 }
		 return result;
	 }

}
