package org.jeecg.modules.projectManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdClinicalProjectMilestonesItems;
import org.jeecg.modules.projectManagement.service.IFdClinicalProjectMilestonesItemsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 临床项目里程碑
 * @author： jeecg-boot
 * @date：   2019-06-03
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdClinicalProjectMilestonesItems")
@Slf4j
public class FdClinicalProjectMilestonesItemsController {
	@Autowired
	private IFdClinicalProjectMilestonesItemsService fdClinicalProjectMilestonesItemsService;
	
	/**
	  * 分页列表查询
	 * @param fdClinicalProjectMilestonesItems
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdClinicalProjectMilestonesItems>> queryPageList(FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItems,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdClinicalProjectMilestonesItems>> result = new Result<IPage<FdClinicalProjectMilestonesItems>>();
		QueryWrapper<FdClinicalProjectMilestonesItems> queryWrapper = QueryGenerator.initQueryWrapper(fdClinicalProjectMilestonesItems, req.getParameterMap());
		Page<FdClinicalProjectMilestonesItems> page = new Page<FdClinicalProjectMilestonesItems>(pageNo, pageSize);
		IPage<FdClinicalProjectMilestonesItems> pageList = fdClinicalProjectMilestonesItemsService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdClinicalProjectMilestonesItems
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	public Result<FdClinicalProjectMilestonesItems> add(@RequestBody @Valid FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItems, BindingResult bindingResult) {
		Result<FdClinicalProjectMilestonesItems> result = new Result<FdClinicalProjectMilestonesItems>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()) {
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			fdClinicalProjectMilestonesItemsService.save(fdClinicalProjectMilestonesItems);
			fdClinicalProjectMilestonesItems.setPkClinicalProjectMilestonesItemId(fdClinicalProjectMilestonesItems.getId());
			fdClinicalProjectMilestonesItemsService.updateById(fdClinicalProjectMilestonesItems);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdClinicalProjectMilestonesItems
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	public Result<FdClinicalProjectMilestonesItems> edit(@RequestBody @Valid FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItems, BindingResult bindingResult) {
		Result<FdClinicalProjectMilestonesItems> result = new Result<FdClinicalProjectMilestonesItems>();
		FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItemsEntity = fdClinicalProjectMilestonesItemsService.getById(fdClinicalProjectMilestonesItems.getId());
		if(fdClinicalProjectMilestonesItemsEntity==null) {
			result.error500("未找到对应实体");
		}else {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()) {
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			boolean ok = fdClinicalProjectMilestonesItemsService.updateById(fdClinicalProjectMilestonesItems);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdClinicalProjectMilestonesItems> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdClinicalProjectMilestonesItems> result = new Result<FdClinicalProjectMilestonesItems>();
		FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItems = fdClinicalProjectMilestonesItemsService.getById(id);
		if(fdClinicalProjectMilestonesItems==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdClinicalProjectMilestonesItemsService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdClinicalProjectMilestonesItems> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdClinicalProjectMilestonesItems> result = new Result<FdClinicalProjectMilestonesItems>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdClinicalProjectMilestonesItemsService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdClinicalProjectMilestonesItems> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdClinicalProjectMilestonesItems> result = new Result<FdClinicalProjectMilestonesItems>();
		FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItems = fdClinicalProjectMilestonesItemsService.getById(id);
		if(fdClinicalProjectMilestonesItems==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdClinicalProjectMilestonesItems);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdClinicalProjectMilestonesItems> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItems = JSON.parseObject(deString, FdClinicalProjectMilestonesItems.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdClinicalProjectMilestonesItems, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdClinicalProjectMilestonesItems> pageList = fdClinicalProjectMilestonesItemsService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "临床项目里程碑列表");
      mv.addObject(NormalExcelConstants.CLASS, FdClinicalProjectMilestonesItems.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("临床项目里程碑列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdClinicalProjectMilestonesItems> listFdClinicalProjectMilestonesItemss = ExcelImportUtil.importExcel(file.getInputStream(), FdClinicalProjectMilestonesItems.class, params);
              for (FdClinicalProjectMilestonesItems fdClinicalProjectMilestonesItemsExcel : listFdClinicalProjectMilestonesItemss) {
                  fdClinicalProjectMilestonesItemsService.save(fdClinicalProjectMilestonesItemsExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdClinicalProjectMilestonesItemss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
