package org.jeecg.modules.projectManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdProjectFileBidparty;
import org.jeecg.modules.projectManagement.service.IFdProjectFileBidpartyService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 项目档案申办方
 * @author： jeecg-boot
 * @date：   2019-06-14
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdProjectFileBidparty")
@Slf4j
public class FdProjectFileBidpartyController {
	@Autowired
	private IFdProjectFileBidpartyService fdProjectFileBidpartyService;
	
	/**
	  * 分页列表查询
	 * @param fdProjectFileBidparty
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdProjectFileBidparty>> queryPageList(FdProjectFileBidparty fdProjectFileBidparty,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdProjectFileBidparty>> result = new Result<IPage<FdProjectFileBidparty>>();
		QueryWrapper<FdProjectFileBidparty> queryWrapper = QueryGenerator.initQueryWrapper(fdProjectFileBidparty, req.getParameterMap());
		Page<FdProjectFileBidparty> page = new Page<FdProjectFileBidparty>(pageNo, pageSize);
		IPage<FdProjectFileBidparty> pageList = fdProjectFileBidpartyService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdProjectFileBidparty
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdProjectFileBidparty> add(@RequestBody FdProjectFileBidparty fdProjectFileBidparty) {
		Result<FdProjectFileBidparty> result = new Result<FdProjectFileBidparty>();
		try {
			fdProjectFileBidpartyService.save(fdProjectFileBidparty);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdProjectFileBidparty
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdProjectFileBidparty> edit(@RequestBody FdProjectFileBidparty fdProjectFileBidparty) {
		Result<FdProjectFileBidparty> result = new Result<FdProjectFileBidparty>();
		FdProjectFileBidparty fdProjectFileBidpartyEntity = fdProjectFileBidpartyService.getById(fdProjectFileBidparty.getId());
		if(fdProjectFileBidpartyEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdProjectFileBidpartyService.updateById(fdProjectFileBidparty);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdProjectFileBidparty> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdProjectFileBidparty> result = new Result<FdProjectFileBidparty>();
		FdProjectFileBidparty fdProjectFileBidparty = fdProjectFileBidpartyService.getById(id);
		if(fdProjectFileBidparty==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdProjectFileBidpartyService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdProjectFileBidparty> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdProjectFileBidparty> result = new Result<FdProjectFileBidparty>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdProjectFileBidpartyService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdProjectFileBidparty> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdProjectFileBidparty> result = new Result<FdProjectFileBidparty>();
		FdProjectFileBidparty fdProjectFileBidparty = fdProjectFileBidpartyService.getById(id);
		if(fdProjectFileBidparty==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdProjectFileBidparty);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdProjectFileBidparty> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdProjectFileBidparty fdProjectFileBidparty = JSON.parseObject(deString, FdProjectFileBidparty.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdProjectFileBidparty, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdProjectFileBidparty> pageList = fdProjectFileBidpartyService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "项目档案申办方列表");
      mv.addObject(NormalExcelConstants.CLASS, FdProjectFileBidparty.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("项目档案申办方列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdProjectFileBidparty> listFdProjectFileBidpartys = ExcelImportUtil.importExcel(file.getInputStream(), FdProjectFileBidparty.class, params);
              for (FdProjectFileBidparty fdProjectFileBidpartyExcel : listFdProjectFileBidpartys) {
                  fdProjectFileBidpartyService.save(fdProjectFileBidpartyExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdProjectFileBidpartys.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
