package org.jeecg.modules.projectManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdProjectFileHospital;
import org.jeecg.modules.projectManagement.service.IFdProjectFileHospitalService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 医院档案表
 * @author： jeecg-boot
 * @date：   2019-05-29
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdProjectFileHospital")
@Slf4j
public class FdProjectFileHospitalController {
	@Autowired
	private IFdProjectFileHospitalService fdProjectFileHospitalService;
	
	/**
	 * @Author gurui
	 * @Description //TODO 分页连接查询
	 * @Date 15:04 2019/5/29
	 * @Param [fdProjectFileHospital, pageNo, pageSize, req]
	 * @return org.jeecg.common.api.vo.Result<com.baomidou.mybatisplus.core.metadata.IPage<org.jeecg.modules.projectManagement.entity.FdProjectFileHospital>>
	 **/
	@GetMapping(value = "/list")
	public Result<IPage<FdProjectFileHospital>> queryPageList(FdProjectFileHospital fdProjectFileHospital,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdProjectFileHospital>> result = new Result<IPage<FdProjectFileHospital>>();
		//对医院名单模糊查询
		if(StringUtils.isNotBlank(fdProjectFileHospital.getHospitalName())){
			fdProjectFileHospital.setHospitalName("*"+fdProjectFileHospital.getHospitalName()+"*");
		}
		QueryWrapper<FdProjectFileHospital> queryWrapper = QueryGenerator.initQueryWrapper(fdProjectFileHospital, req.getParameterMap());
		Page<FdProjectFileHospital> page = new Page<FdProjectFileHospital>(pageNo, pageSize);
		IPage<FdProjectFileHospital> pageList = fdProjectFileHospitalService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdProjectFileHospital
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdProjectFileHospital> add(@RequestBody FdProjectFileHospital fdProjectFileHospital) {
		Result<FdProjectFileHospital> result = new Result<FdProjectFileHospital>();
		try {
			fdProjectFileHospitalService.save(fdProjectFileHospital);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdProjectFileHospital
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdProjectFileHospital> edit(@RequestBody FdProjectFileHospital fdProjectFileHospital) {
		Result<FdProjectFileHospital> result = new Result<FdProjectFileHospital>();
		FdProjectFileHospital fdProjectFileHospitalEntity = fdProjectFileHospitalService.getById(fdProjectFileHospital.getId());
		if(fdProjectFileHospitalEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdProjectFileHospitalService.updateById(fdProjectFileHospital);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdProjectFileHospital> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdProjectFileHospital> result = new Result<FdProjectFileHospital>();
		FdProjectFileHospital fdProjectFileHospital = fdProjectFileHospitalService.getById(id);
		if(fdProjectFileHospital==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdProjectFileHospitalService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdProjectFileHospital> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdProjectFileHospital> result = new Result<FdProjectFileHospital>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdProjectFileHospitalService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdProjectFileHospital> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdProjectFileHospital> result = new Result<FdProjectFileHospital>();
		FdProjectFileHospital fdProjectFileHospital = fdProjectFileHospitalService.getById(id);
		if(fdProjectFileHospital==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdProjectFileHospital);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdProjectFileHospital> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdProjectFileHospital fdProjectFileHospital = JSON.parseObject(deString, FdProjectFileHospital.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdProjectFileHospital, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdProjectFileHospital> pageList = fdProjectFileHospitalService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "医院档案表列表");
      mv.addObject(NormalExcelConstants.CLASS, FdProjectFileHospital.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("医院档案表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdProjectFileHospital> listFdProjectFileHospitals = ExcelImportUtil.importExcel(file.getInputStream(), FdProjectFileHospital.class, params);
              for (FdProjectFileHospital fdProjectFileHospitalExcel : listFdProjectFileHospitals) {
                  fdProjectFileHospitalService.save(fdProjectFileHospitalExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdProjectFileHospitals.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
