package org.jeecg.modules.projectManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdProjectFileInvoice;
import org.jeecg.modules.projectManagement.service.IFdProjectFileInvoiceService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 支付信息
 * @author： jeecg-boot
 * @date：   2020-07-21
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdProjectFileInvoice")
@Slf4j
public class FdProjectFileInvoiceController {
	@Autowired
	private IFdProjectFileInvoiceService fdProjectFileInvoiceService;
	
	/**
	  * 分页列表查询
	 * @param fdProjectFileInvoice
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdProjectFileInvoice>> queryPageList(FdProjectFileInvoice fdProjectFileInvoice,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdProjectFileInvoice>> result = new Result<IPage<FdProjectFileInvoice>>();
		QueryWrapper<FdProjectFileInvoice> queryWrapper = QueryGenerator.initQueryWrapper(fdProjectFileInvoice, req.getParameterMap());
		queryWrapper.lambda().eq(FdProjectFileInvoice::getDelFlag,"0");
		queryWrapper.lambda().orderByAsc(FdProjectFileInvoice::getSort);
		Page<FdProjectFileInvoice> page = new Page<FdProjectFileInvoice>(pageNo, pageSize);
		IPage<FdProjectFileInvoice> pageList = fdProjectFileInvoiceService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdProjectFileInvoice
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdProjectFileInvoice> add(@RequestBody FdProjectFileInvoice fdProjectFileInvoice) {
		Result<FdProjectFileInvoice> result = new Result<FdProjectFileInvoice>();
		try {
			fdProjectFileInvoiceService.save(fdProjectFileInvoice);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdProjectFileInvoice
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdProjectFileInvoice> edit(@RequestBody FdProjectFileInvoice fdProjectFileInvoice) {
		Result<FdProjectFileInvoice> result = new Result<FdProjectFileInvoice>();
		FdProjectFileInvoice fdProjectFileInvoiceEntity = fdProjectFileInvoiceService.getById(fdProjectFileInvoice.getId());
		if(fdProjectFileInvoiceEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdProjectFileInvoiceService.updateById(fdProjectFileInvoice);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdProjectFileInvoice> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdProjectFileInvoice> result = new Result<FdProjectFileInvoice>();
		FdProjectFileInvoice fdProjectFileInvoice = fdProjectFileInvoiceService.getById(id);
		if(fdProjectFileInvoice==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdProjectFileInvoiceService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdProjectFileInvoice> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdProjectFileInvoice> result = new Result<FdProjectFileInvoice>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdProjectFileInvoiceService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdProjectFileInvoice> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdProjectFileInvoice> result = new Result<FdProjectFileInvoice>();
		FdProjectFileInvoice fdProjectFileInvoice = fdProjectFileInvoiceService.getById(id);
		if(fdProjectFileInvoice==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdProjectFileInvoice);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdProjectFileInvoice> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdProjectFileInvoice fdProjectFileInvoice = JSON.parseObject(deString, FdProjectFileInvoice.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdProjectFileInvoice, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdProjectFileInvoice> pageList = fdProjectFileInvoiceService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "支付信息列表");
      mv.addObject(NormalExcelConstants.CLASS, FdProjectFileInvoice.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("支付信息列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdProjectFileInvoice> listFdProjectFileInvoices = ExcelImportUtil.importExcel(file.getInputStream(), FdProjectFileInvoice.class, params);
              for (FdProjectFileInvoice fdProjectFileInvoiceExcel : listFdProjectFileInvoices) {
                  fdProjectFileInvoiceService.save(fdProjectFileInvoiceExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdProjectFileInvoices.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

  /**
  * @Author chenchuang
  * @Description //TODO 保存全部
  * @Date 2020/7/22 16:07
  * @Param [fdProjectFileInvoiceList]
  * @return org.jeecg.common.api.vo.Result
  */
  @PostMapping(value = "/addAll")
  public Result addAll(@RequestBody List<FdProjectFileInvoice> fdProjectFileInvoiceList){
  	Result result=new Result();
	  try {
		  fdProjectFileInvoiceService.addAll(fdProjectFileInvoiceList);
		  result.success("操作成功");
	  } catch (Exception e) {
		  e.printStackTrace();
		  result.error500("操作失败："+e.getMessage());
	  }
	  return result;
  }

}
