package org.jeecg.modules.projectManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdProjectQuotationDetail;
import org.jeecg.modules.projectManagement.service.IFdProjectQuotationDetailService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 项目报价明细
 * @author： jeecg-boot
 * @date： 2019-05-20
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdProjectQuotationDetail")
@Slf4j
public class FdProjectQuotationDetailController {
    @Autowired
    private IFdProjectQuotationDetailService fdProjectQuotationDetailService;

    /**
     * 分页列表查询
     *
     * @param fdProjectQuotationDetail
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdProjectQuotationDetail>> queryPageList(FdProjectQuotationDetail fdProjectQuotationDetail,
                                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                 HttpServletRequest req) {
        Result<IPage<FdProjectQuotationDetail>> result = new Result<IPage<FdProjectQuotationDetail>>();
        QueryWrapper<FdProjectQuotationDetail> queryWrapper = QueryGenerator.initQueryWrapper(fdProjectQuotationDetail, req.getParameterMap());
        queryWrapper.lambda().orderByAsc(FdProjectQuotationDetail::getSort);
        queryWrapper.eq("del_flag","0");
        Page<FdProjectQuotationDetail> page = new Page<FdProjectQuotationDetail>(pageNo, pageSize);
        IPage<FdProjectQuotationDetail> pageList = fdProjectQuotationDetailService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdProjectQuotationDetail
     * @return
     */
    @PostMapping(value = "/add")
    public Result<FdProjectQuotationDetail> add(@RequestBody FdProjectQuotationDetail fdProjectQuotationDetail) {
        Result<FdProjectQuotationDetail> result = new Result<FdProjectQuotationDetail>();
        try {
            fdProjectQuotationDetailService.save(fdProjectQuotationDetail);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdProjectQuotationDetail
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<FdProjectQuotationDetail> edit(@RequestBody FdProjectQuotationDetail fdProjectQuotationDetail) {
        Result<FdProjectQuotationDetail> result = new Result<FdProjectQuotationDetail>();
        FdProjectQuotationDetail fdProjectQuotationDetailEntity = fdProjectQuotationDetailService.getById(fdProjectQuotationDetail.getId());
        if (fdProjectQuotationDetailEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdProjectQuotationDetailService.updateById(fdProjectQuotationDetail);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<FdProjectQuotationDetail> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdProjectQuotationDetail> result = new Result<FdProjectQuotationDetail>();
        FdProjectQuotationDetail fdProjectQuotationDetail = fdProjectQuotationDetailService.getById(id);
        if (fdProjectQuotationDetail == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdProjectQuotationDetailService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdProjectQuotationDetail> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdProjectQuotationDetail> result = new Result<FdProjectQuotationDetail>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdProjectQuotationDetailService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdProjectQuotationDetail> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdProjectQuotationDetail> result = new Result<FdProjectQuotationDetail>();
        FdProjectQuotationDetail fdProjectQuotationDetail = fdProjectQuotationDetailService.getById(id);
        if (fdProjectQuotationDetail == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdProjectQuotationDetail);
            result.setSuccess(true);
        }
        return result;
    }

    // @GetMapping(value = "/queryByQuotationId")
    // public Result<FdProjectQuotationDetail> queryByQuotationId(@RequestParam(name = "quotationId", required = true) String quotationId) {
    //     Result<FdProjectQuotationDetail> result = new Result<FdProjectQuotationDetail>();
    //     LambdaQueryWrapper<FdProjectQuotationDetail> lambdaQueryWrapper = new LambdaQueryWrapper<>();
    //     lambdaQueryWrapper.eq(FdProjectQuotationDetail::getQuotationId, quotationId);
    //     FdProjectQuotationDetail fdProjectQuotationDetail=new FdProjectQuotationDetail();
    //     for (FdProjectQuotationDetail de : fdProjectQuotationDetailService.list(lambdaQueryWrapper)) {
    //         fdProjectQuotationDetail=de;
    //     }
    //     if (fdProjectQuotationDetail == null) {
    //         result.error500("未找到对应实体");
    //     } else {
    //         result.setResult(fdProjectQuotationDetail);
    //         result.setSuccess(true);
    //     }
    //     return result;
    // }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdProjectQuotationDetail> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdProjectQuotationDetail fdProjectQuotationDetail = JSON.parseObject(deString, FdProjectQuotationDetail.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdProjectQuotationDetail, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdProjectQuotationDetail> pageList = fdProjectQuotationDetailService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "项目报价明细列表");
        mv.addObject(NormalExcelConstants.CLASS, FdProjectQuotationDetail.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("项目报价明细列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdProjectQuotationDetail> listFdProjectQuotationDetails = ExcelImportUtil.importExcel(file.getInputStream(), FdProjectQuotationDetail.class, params);
                for (FdProjectQuotationDetail fdProjectQuotationDetailExcel : listFdProjectQuotationDetails) {
                    fdProjectQuotationDetailService.save(fdProjectQuotationDetailExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdProjectQuotationDetails.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

}
