package org.jeecg.modules.projectManagement.controller;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdProjectQuotationTemplateItem;
import org.jeecg.modules.projectManagement.entity.FdProjectQuotationTemplateItemTree;
import org.jeecg.modules.projectManagement.service.IFdProjectQuotationTemplateItemService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 项目报价模板对照明细
 * @author： jeecg-boot
 * @date： 2019-05-20
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdProjectQuotationTemplateItem")
@Slf4j
public class FdProjectQuotationTemplateItemController {
    @Autowired
    private IFdProjectQuotationTemplateItemService fdProjectQuotationTemplateItemService;

    /**
     * 分页列表查询
     *
     * @param fdProjectQuotationTemplateItem
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdProjectQuotationTemplateItem>> queryPageList(FdProjectQuotationTemplateItem fdProjectQuotationTemplateItem,
                                                                       @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                       @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                       HttpServletRequest req) {
        Result<IPage<FdProjectQuotationTemplateItem>> result = new Result<IPage<FdProjectQuotationTemplateItem>>();
        QueryWrapper<FdProjectQuotationTemplateItem> queryWrapper = QueryGenerator.initQueryWrapper(fdProjectQuotationTemplateItem, req.getParameterMap());
        queryWrapper.lambda().eq(FdProjectQuotationTemplateItem::getDelFlag,"0");
        Page<FdProjectQuotationTemplateItem> page = new Page<FdProjectQuotationTemplateItem>(pageNo, pageSize);
        IPage<FdProjectQuotationTemplateItem> pageList = fdProjectQuotationTemplateItemService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdProjectQuotationTemplateItem
     * @return
     */
    @PostMapping(value = "/add")
    public Result<FdProjectQuotationTemplateItem> add(@RequestBody FdProjectQuotationTemplateItem fdProjectQuotationTemplateItem) {
        Result<FdProjectQuotationTemplateItem> result = new Result<FdProjectQuotationTemplateItem>();
        try {
            fdProjectQuotationTemplateItemService.saveProjectQuotationTemplateItem(fdProjectQuotationTemplateItem);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdProjectQuotationTemplateItem
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<FdProjectQuotationTemplateItem> edit(@RequestBody FdProjectQuotationTemplateItem fdProjectQuotationTemplateItem) {
        Result<FdProjectQuotationTemplateItem> result = new Result<FdProjectQuotationTemplateItem>();
        FdProjectQuotationTemplateItem fdProjectQuotationTemplateItemEntity = fdProjectQuotationTemplateItemService.getById(fdProjectQuotationTemplateItem.getId());
        if (fdProjectQuotationTemplateItemEntity == null) {
            result.error500("未找到对应实体");
        } else {
            fdProjectQuotationTemplateItemService.updateProjectQuotationTemplateItem(fdProjectQuotationTemplateItem);

        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<FdProjectQuotationTemplateItem> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdProjectQuotationTemplateItem> result = new Result<FdProjectQuotationTemplateItem>();
        FdProjectQuotationTemplateItem fdProjectQuotationTemplateItem = fdProjectQuotationTemplateItemService.getById(id);
        if (fdProjectQuotationTemplateItem == null) {
            result.error500("未找到对应实体");
        } else {
            fdProjectQuotationTemplateItem.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
            fdProjectQuotationTemplateItemService.updateById(fdProjectQuotationTemplateItem);

        }

        return result;
    }

    /**
     * @Author gurui
     * @Description //TODO 单个或多个删除
     * @Date 13:27 2019/5/21
     * @Param [ids]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.projectManagement.entity.FdProjectQuotationTemplateItem>
     **/
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdProjectQuotationTemplateItem> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdProjectQuotationTemplateItem> result = new Result<FdProjectQuotationTemplateItem>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            fdProjectQuotationTemplateItemService.deleteProjectQuotationTemplateItem(ids);
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdProjectQuotationTemplateItem> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdProjectQuotationTemplateItem> result = new Result<FdProjectQuotationTemplateItem>();
        FdProjectQuotationTemplateItem fdProjectQuotationTemplateItem = fdProjectQuotationTemplateItemService.getById(id);
        if (fdProjectQuotationTemplateItem == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdProjectQuotationTemplateItem);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdProjectQuotationTemplateItem> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdProjectQuotationTemplateItem fdProjectQuotationTemplateItem = JSON.parseObject(deString, FdProjectQuotationTemplateItem.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdProjectQuotationTemplateItem, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdProjectQuotationTemplateItem> pageList = fdProjectQuotationTemplateItemService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "项目报价模板对照明细列表");
        mv.addObject(NormalExcelConstants.CLASS, FdProjectQuotationTemplateItem.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("项目报价模板对照明细列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdProjectQuotationTemplateItem> listFdProjectQuotationTemplateItems = ExcelImportUtil.importExcel(file.getInputStream(), FdProjectQuotationTemplateItem.class, params);
                for (FdProjectQuotationTemplateItem fdProjectQuotationTemplateItemExcel : listFdProjectQuotationTemplateItems) {
                    fdProjectQuotationTemplateItemService.save(fdProjectQuotationTemplateItemExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdProjectQuotationTemplateItems.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }


    @GetMapping(value = "/getListMerge")
    public Result<List<FdProjectQuotationTemplateItem>> getListMerge(){
        Result<List<FdProjectQuotationTemplateItem>> result=new Result<>();

        // QueryWrapper<FdProjectQuotationTemplateItem> lambdaQueryWrapper=new QueryWrapper<>();
        // lambdaQueryWrapper.eq("","576207d726b0d5a944c3feaa86b35eff");
        // lambdaQueryWrapper.eq("","6654a5020536215e3f6c9d5bca8a8f46");

        List<FdProjectQuotationTemplateItem> listProjectTemplateItem=fdProjectQuotationTemplateItemService.getListTemplateItem();
        List<Map<String,Object>> mergeMap=fdProjectQuotationTemplateItemService.getTemplateItemMerge();
        String templateItemIdEquale="";
        for(FdProjectQuotationTemplateItem projectQuotationTemplateItem:listProjectTemplateItem){
            String templateItemId=projectQuotationTemplateItem.getProjectQuotationTemplateId();//报价单模板对照id
            if(!templateItemIdEquale.equals(templateItemId)){
                for(Map<String,Object> map:mergeMap){
                    if(templateItemId.equals(map.get("id"))){
                        projectQuotationTemplateItem.setCount(Integer.parseInt(map.get("count").toString()));
                        templateItemIdEquale=templateItemId;
                        break;
                    }
                }
            }

        }
        result.setResult(listProjectTemplateItem);
        return result;
    }

    /***
     * @Title:查询报价模板
            * @Description:TODO
            * @param pkTemplateId, quotation] 【模板id,报价id】
            * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.projectManagement.entity.FdProjectQuotationTemplateItemTree>>
            * @exception
            * @author 陈创
            * @date 2019/6/13 11:39
            * @updatehistory:
     *   2019/6/13 11:39 新增
            */
    @GetMapping(value = "/queryTreeList")
    public Result<List<FdProjectQuotationTemplateItemTree>> queryTreeList(@RequestParam(name="pkTemplateId") String pkTemplateId,@RequestParam("quotation") String quotation){
        Result<List<FdProjectQuotationTemplateItemTree>> result = new Result<>();
        try {
            LambdaQueryWrapper<FdProjectQuotationTemplateItem> lambdaQueryWrapper =new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(FdProjectQuotationTemplateItem::getDelFlag,"0");
            lambdaQueryWrapper.eq(FdProjectQuotationTemplateItem::getProjectQuotationTemplateId,pkTemplateId);
            lambdaQueryWrapper.eq(FdProjectQuotationTemplateItem::getQuotationId,quotation);
            // lambdaQueryWrapper.orderBy(true,true,FdProjectQuotationTemplateItem::getCode);
            List<FdProjectQuotationTemplateItem> list = fdProjectQuotationTemplateItemService.listOrderByCode(quotation,pkTemplateId);
            List<FdProjectQuotationTemplateItemTree> treeLists = new ArrayList<>();
            getTreeList(treeLists,list,null);
            findLeaf(treeLists);
            result.setResult(treeLists);
        }catch (Exception e){
            e.fillInStackTrace();
        }
        return result;
    }

    private void getTreeList(List<FdProjectQuotationTemplateItemTree> treeList,List<FdProjectQuotationTemplateItem> fdlist ,FdProjectQuotationTemplateItemTree temp){
        for (FdProjectQuotationTemplateItem fdQuotationTemplateItem: fdlist){
            String tempPid = fdQuotationTemplateItem.getParentId();
            FdProjectQuotationTemplateItemTree tree = new FdProjectQuotationTemplateItemTree(fdQuotationTemplateItem);
            if (temp == null && oConvertUtils.isEmpty(tempPid)){
                treeList.add(tree);
                if (!tree.getIsLeaf()){
                    getTreeList(treeList,fdlist,tree);
                }
            }else  if (temp!=null&&tempPid!=null&&tempPid.equals(temp.getId())){
                temp.getChildren().add(tree);
                if (!tree.getIsLeaf()){
                    getTreeList(treeList,fdlist,tree);
                }
            }

        }
    }

    private void findLeaf(List<FdProjectQuotationTemplateItemTree> treeList) {
        for (FdProjectQuotationTemplateItemTree fdQuotationTemplateItemTree : treeList) {
            if (fdQuotationTemplateItemTree.getChildren().isEmpty()) {
                fdQuotationTemplateItemTree.setIsLeaf(true);
                fdQuotationTemplateItemTree.setChildren(null);
            } else {
                findLeaf(fdQuotationTemplateItemTree.getChildren());
            }
        }
    }

}
