package org.jeecg.modules.projectManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdProjectQuotation;
import org.jeecg.modules.projectManagement.entity.FdProjectQuotationUpload;
import org.jeecg.modules.projectManagement.service.IFdProjectQuotationUploadService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 项目报价上传附件表
 * @author： jeecg-boot
 * @date： 2019-09-06
 * @version： V1.0
 */
@RestController
@RequestMapping("/projectManagement/fdProjectQuotationUpload")
@Slf4j
public class FdProjectQuotationUploadController {
    @Autowired
    private IFdProjectQuotationUploadService fdProjectQuotationUploadService;

    /**
     * 分页列表查询
     *
     * @param fdProjectQuotationUpload
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdProjectQuotationUpload>> queryPageList(FdProjectQuotationUpload fdProjectQuotationUpload,
                                                                 @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                                 @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                                 HttpServletRequest req) {
        Result<IPage<FdProjectQuotationUpload>> result = new Result<IPage<FdProjectQuotationUpload>>();
        QueryWrapper<FdProjectQuotationUpload> queryWrapper = QueryGenerator.initQueryWrapper(fdProjectQuotationUpload, req.getParameterMap());
        queryWrapper.lambda().eq(FdProjectQuotationUpload::getDelFlag,"0");
        queryWrapper.lambda().orderByAsc(FdProjectQuotationUpload::getSort);
        Page<FdProjectQuotationUpload> page = new Page<FdProjectQuotationUpload>(pageNo, pageSize);
        IPage<FdProjectQuotationUpload> pageList = fdProjectQuotationUploadService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdProjectQuotationUpload
     * @return
     */
    @PostMapping(value = "/add")
    public Result<FdProjectQuotationUpload> add(@RequestBody FdProjectQuotationUpload fdProjectQuotationUpload) {
        Result<FdProjectQuotationUpload> result = new Result<FdProjectQuotationUpload>();
        try {
            fdProjectQuotationUploadService.save(fdProjectQuotationUpload);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdProjectQuotationUpload
     * @return
     */
    @PutMapping(value = "/edit")
    public Result<FdProjectQuotationUpload> edit(@RequestBody FdProjectQuotationUpload fdProjectQuotationUpload) {
        Result<FdProjectQuotationUpload> result = new Result<FdProjectQuotationUpload>();
        FdProjectQuotationUpload fdProjectQuotationUploadEntity = fdProjectQuotationUploadService.getById(fdProjectQuotationUpload.getId());
        if (fdProjectQuotationUploadEntity == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdProjectQuotationUploadService.updateById(fdProjectQuotationUpload);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<FdProjectQuotationUpload> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdProjectQuotationUpload> result = new Result<FdProjectQuotationUpload>();
        FdProjectQuotationUpload fdProjectQuotationUpload = fdProjectQuotationUploadService.getById(id);
        if (fdProjectQuotationUpload == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdProjectQuotationUploadService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdProjectQuotationUpload> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdProjectQuotationUpload> result = new Result<FdProjectQuotationUpload>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdProjectQuotationUploadService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdProjectQuotationUpload> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdProjectQuotationUpload> result = new Result<FdProjectQuotationUpload>();
        FdProjectQuotationUpload fdProjectQuotationUpload = fdProjectQuotationUploadService.getById(id);
        if (fdProjectQuotationUpload == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdProjectQuotationUpload);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdProjectQuotationUpload> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdProjectQuotationUpload fdProjectQuotationUpload = JSON.parseObject(deString, FdProjectQuotationUpload.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdProjectQuotationUpload, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdProjectQuotationUpload> pageList = fdProjectQuotationUploadService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "项目报价上传附件表列表");
        mv.addObject(NormalExcelConstants.CLASS, FdProjectQuotationUpload.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("项目报价上传附件表列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdProjectQuotationUpload> listFdProjectQuotationUploads = ExcelImportUtil.importExcel(file.getInputStream(), FdProjectQuotationUpload.class, params);
                for (FdProjectQuotationUpload fdProjectQuotationUploadExcel : listFdProjectQuotationUploads) {
                    fdProjectQuotationUploadService.save(fdProjectQuotationUploadExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdProjectQuotationUploads.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /***
     * @Title:批量保存
     * @Description:TODO
     * @param fdProjectQuotation
     * @return org.jeecg.common.api.vo.Result
     * @exception
     * @author 陈创
     * @date 2019/9/6 15:47
     * @updatehistory:
     *   2019/9/6 15:47 新增
     */
    @PostMapping(value = "/batchUpload")
    public Result batchUpload(@RequestBody FdProjectQuotation fdProjectQuotation) {
        Result result = new Result();
        try {
            //批量删除
            LambdaQueryWrapper<FdProjectQuotationUpload> fdProjectQuotationUploadLambdaQueryWrapper=new LambdaQueryWrapper<>();
            fdProjectQuotationUploadLambdaQueryWrapper.eq(FdProjectQuotationUpload::getProjectId,fdProjectQuotation.getId());
            fdProjectQuotationUploadService.remove(fdProjectQuotationUploadLambdaQueryWrapper);

            //改变报价主表是否上传文件状态
            fdProjectQuotationUploadService.quotationIsUpload(fdProjectQuotation);

            //批量保存
            fdProjectQuotationUploadService.saveBatch(fdProjectQuotation.getFdProjectQuotationUploadList());
            result.setSuccess(true);
        } catch (Exception e) {
            e.printStackTrace();
            result.error500("操作失败" + e.getMessage());
        }
        return result;
    }

}
