package org.jeecg.modules.qualityControl.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.qualityControl.entity.FdQaProcessFacilityItems;
import org.jeecg.modules.qualityControl.service.IFdQaProcessFacilityItemsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.qualityControl.service.IFdQaProcessFacilityService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 质量保证（工艺与设备）明细
 * @author： jeecg-boot
 * @date：   2019-06-06
 * @version： V1.0
 */
@RestController
@RequestMapping("/qualityControl/fdQaProcessFacilityItems")
@Slf4j
public class FdQaProcessFacilityItemsController {
	@Autowired
	private IFdQaProcessFacilityItemsService fdQaProcessFacilityItemsService;

	@Autowired
	private IFdQaProcessFacilityService iFdQaProcessFacilityService;

	/**
	  * 分页列表查询
	 * @param fdQaProcessFacilityItems
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdQaProcessFacilityItems>> queryPageList(FdQaProcessFacilityItems fdQaProcessFacilityItems,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdQaProcessFacilityItems>> result = new Result<IPage<FdQaProcessFacilityItems>>();
		QueryWrapper<FdQaProcessFacilityItems> queryWrapper = QueryGenerator.initQueryWrapper(fdQaProcessFacilityItems, req.getParameterMap());
		queryWrapper.lambda().orderByAsc(FdQaProcessFacilityItems::getSort);
		Page<FdQaProcessFacilityItems> page = new Page<FdQaProcessFacilityItems>(pageNo, pageSize);
		IPage<FdQaProcessFacilityItems> pageList = fdQaProcessFacilityItemsService.page(page, queryWrapper);
		//获取对应图片集合
		iFdQaProcessFacilityService.getPictureUpload(pageList.getRecords());
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdQaProcessFacilityItems
	 * @return
	 */
	@PostMapping(value = "/add")
	@NotDuplicate
	public Result<FdQaProcessFacilityItems> add(@RequestBody @Valid FdQaProcessFacilityItems fdQaProcessFacilityItems , BindingResult bindingResult) {
		Result<FdQaProcessFacilityItems> result = new Result<FdQaProcessFacilityItems>();
		try {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()) {
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			fdQaProcessFacilityItemsService.save(fdQaProcessFacilityItems);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdQaProcessFacilityItems
	 * @return
	 */
	@PutMapping(value = "/edit")
	@NotDuplicate
	public Result<FdQaProcessFacilityItems> edit(@RequestBody @Valid FdQaProcessFacilityItems fdQaProcessFacilityItems, BindingResult bindingResult) {
		Result<FdQaProcessFacilityItems> result = new Result<FdQaProcessFacilityItems>();
		FdQaProcessFacilityItems fdQaProcessFacilityItemsEntity = fdQaProcessFacilityItemsService.getById(fdQaProcessFacilityItems.getId());
		if(fdQaProcessFacilityItemsEntity==null) {
			result.error500("未找到对应实体");
		}else {
			StringBuilder sb = new StringBuilder();
			if (bindingResult.hasErrors()) {
				//记录错误信息
				bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
				result.error500(sb.toString());
				return result;
			}
			boolean ok = fdQaProcessFacilityItemsService.updateById(fdQaProcessFacilityItems);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdQaProcessFacilityItems> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdQaProcessFacilityItems> result = new Result<FdQaProcessFacilityItems>();
		FdQaProcessFacilityItems fdQaProcessFacilityItems = fdQaProcessFacilityItemsService.getById(id);
		if(fdQaProcessFacilityItems==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdQaProcessFacilityItemsService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdQaProcessFacilityItems> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdQaProcessFacilityItems> result = new Result<FdQaProcessFacilityItems>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdQaProcessFacilityItemsService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdQaProcessFacilityItems> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdQaProcessFacilityItems> result = new Result<FdQaProcessFacilityItems>();
		FdQaProcessFacilityItems fdQaProcessFacilityItems = fdQaProcessFacilityItemsService.getById(id);
		if(fdQaProcessFacilityItems==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdQaProcessFacilityItems);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdQaProcessFacilityItems> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdQaProcessFacilityItems fdQaProcessFacilityItems = JSON.parseObject(deString, FdQaProcessFacilityItems.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdQaProcessFacilityItems, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdQaProcessFacilityItems> pageList = fdQaProcessFacilityItemsService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "质量保证（工艺与设备）明细列表");
      mv.addObject(NormalExcelConstants.CLASS, FdQaProcessFacilityItems.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("质量保证（工艺与设备）明细列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdQaProcessFacilityItems> listFdQaProcessFacilityItemss = ExcelImportUtil.importExcel(file.getInputStream(), FdQaProcessFacilityItems.class, params);
              for (FdQaProcessFacilityItems fdQaProcessFacilityItemsExcel : listFdQaProcessFacilityItemss) {
                  fdQaProcessFacilityItemsService.save(fdQaProcessFacilityItemsExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdQaProcessFacilityItemss.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
