package org.jeecg.modules.qualityControl.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.qualityControl.entity.FdQcSample;
import org.jeecg.modules.qualityControl.service.IFdQcSampleService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

 /**
 * @Title: Controller
 * @Description: 质量控制生物样本分析表
 * @author： jeecg-boot
 * @date：   2019-07-22
 * @version： V1.0
 */
@RestController
@RequestMapping("/qualityControl/fdQcSample")
@Slf4j
public class FdQcSampleController {
	@Autowired
	private IFdQcSampleService fdQcSampleService;
	
	/**
	  * 分页列表查询
	 * @param fdQcSample
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@GetMapping(value = "/list")
	public Result<IPage<FdQcSample>> queryPageList(FdQcSample fdQcSample,
									  @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
									  @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
									  HttpServletRequest req) {
		Result<IPage<FdQcSample>> result = new Result<IPage<FdQcSample>>();
		QueryWrapper<FdQcSample> queryWrapper = QueryGenerator.initQueryWrapper(fdQcSample, req.getParameterMap());
		Page<FdQcSample> page = new Page<FdQcSample>(pageNo, pageSize);
		IPage<FdQcSample> pageList = fdQcSampleService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}
	
	/**
	  *   添加
	 * @param fdQcSample
	 * @return
	 */
	@PostMapping(value = "/add")
	public Result<FdQcSample> add(@RequestBody FdQcSample fdQcSample) {
		Result<FdQcSample> result = new Result<FdQcSample>();
		try {
			fdQcSampleService.save(fdQcSample);
			result.success("添加成功！");
		} catch (Exception e) {
			e.printStackTrace();
			log.info(e.getMessage());
			result.error500("操作失败");
		}
		return result;
	}
	
	/**
	  *  编辑
	 * @param fdQcSample
	 * @return
	 */
	@PutMapping(value = "/edit")
	public Result<FdQcSample> edit(@RequestBody FdQcSample fdQcSample) {
		Result<FdQcSample> result = new Result<FdQcSample>();
		FdQcSample fdQcSampleEntity = fdQcSampleService.getById(fdQcSample.getId());
		if(fdQcSampleEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdQcSampleService.updateById(fdQcSample);
			//TODO 返回false说明什么？
			if(ok) {
				result.success("修改成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@DeleteMapping(value = "/delete")
	public Result<FdQcSample> delete(@RequestParam(name="id",required=true) String id) {
		Result<FdQcSample> result = new Result<FdQcSample>();
		FdQcSample fdQcSample = fdQcSampleService.getById(id);
		if(fdQcSample==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = fdQcSampleService.removeById(id);
			if(ok) {
				result.success("删除成功!");
			}
		}
		
		return result;
	}
	
	/**
	  *  批量删除
	 * @param ids
	 * @return
	 */
	@DeleteMapping(value = "/deleteBatch")
	public Result<FdQcSample> deleteBatch(@RequestParam(name="ids",required=true) String ids) {
		Result<FdQcSample> result = new Result<FdQcSample>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.fdQcSampleService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}
	
	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@GetMapping(value = "/queryById")
	public Result<FdQcSample> queryById(@RequestParam(name="id",required=true) String id) {
		Result<FdQcSample> result = new Result<FdQcSample>();
		FdQcSample fdQcSample = fdQcSampleService.getById(id);
		if(fdQcSample==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(fdQcSample);
			result.setSuccess(true);
		}
		return result;
	}

  /**
      * 导出excel
   *
   * @param request
   * @param response
   */
  @RequestMapping(value = "/exportXls")
  public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
      // Step.1 组装查询条件
      QueryWrapper<FdQcSample> queryWrapper = null;
      try {
          String paramsStr = request.getParameter("paramsStr");
          if (oConvertUtils.isNotEmpty(paramsStr)) {
              String deString = URLDecoder.decode(paramsStr, "UTF-8");
              FdQcSample fdQcSample = JSON.parseObject(deString, FdQcSample.class);
              queryWrapper = QueryGenerator.initQueryWrapper(fdQcSample, request.getParameterMap());
          }
      } catch (UnsupportedEncodingException e) {
          e.printStackTrace();
      }

      //Step.2 AutoPoi 导出Excel
      ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
      List<FdQcSample> pageList = fdQcSampleService.list(queryWrapper);
      //导出文件名称
      mv.addObject(NormalExcelConstants.FILE_NAME, "质量控制生物样本分析表列表");
      mv.addObject(NormalExcelConstants.CLASS, FdQcSample.class);
      mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("质量控制生物样本分析表列表数据", "导出人:Jeecg", "导出信息"));
      mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
      return mv;
  }

  /**
      * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();// 获取上传文件对象
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<FdQcSample> listFdQcSamples = ExcelImportUtil.importExcel(file.getInputStream(), FdQcSample.class, params);
              for (FdQcSample fdQcSampleExcel : listFdQcSamples) {
                  fdQcSampleService.save(fdQcSampleExcel);
              }
              return Result.ok("文件导入成功！数据行数：" + listFdQcSamples.size());
          } catch (Exception e) {
              log.error(e.getMessage());
              return Result.error("文件导入失败！");
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
