package org.jeecg.modules.sampleAndConsumableManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdHospital;
import org.jeecg.modules.basedata.entity.FdMatrixType;
import org.jeecg.modules.basedata.mapper.FdHospitalMapper;
import org.jeecg.modules.basedata.mapper.FdMatrixTypeMapper;
import org.jeecg.modules.basedata.service.IFdHospitalService;
import org.jeecg.modules.projectManagement.entity.FdContract;
import org.jeecg.modules.projectManagement.entity.FdProjectFile;
import org.jeecg.modules.projectManagement.entity.FdProjectFileMethod;
import org.jeecg.modules.projectManagement.mapper.FdProjectFileMapper;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrate;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrateFileUpload;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrateRecipientRecord;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdBlankSubstrateFileUploadService;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdBlankSubstrateRecipientRecordService;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdBlankSubstrateService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 空白基质管理
 * @author： jeecg-boot
 * @date： 2019-06-10
 * @version： V1.0
 */
@RestController
@RequestMapping("/sampleAndConsumableManagement/fdBlankSubstrate")
@Slf4j
public class FdBlankSubstrateController {
    @Autowired
    private IFdBlankSubstrateService fdBlankSubstrateService;

    @Resource
    private FdHospitalMapper fdHospitalMapper;

    @Resource
    private FdMatrixTypeMapper fdMatrixTypeMapper;

    @Resource
    private IFdBlankSubstrateRecipientRecordService fdBlankSubstrateRecipientRecordService;

    @Resource
    private IFdBlankSubstrateFileUploadService fdBlankSubstrateFileUploadService;


    /**
     * 分页列表查询
     *
     * @param fdBlankSubstrate
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    @RequiresPermissions("url:blank-substrate:view")
    public Result<IPage<FdBlankSubstrate>> queryPageList(FdBlankSubstrate fdBlankSubstrate,
                                                         @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                         @RequestParam(name = "pageSize", defaultValue = "300") Integer pageSize,
                                                         HttpServletRequest req) {
        Result<IPage<FdBlankSubstrate>> result = new Result<IPage<FdBlankSubstrate>>();
        String trackingCode = fdBlankSubstrate.getTrackingCode();
        if (StringUtils.isNotBlank(fdBlankSubstrate.getTrackingCode())) {
            fdBlankSubstrate.setTrackingCode(null);
        }
        String contractCode = fdBlankSubstrate.getContractCode();
        if (StringUtils.isNotBlank(fdBlankSubstrate.getContractCode())) {
            fdBlankSubstrate.setContractCode(null);
        }
        //年份
        String year=fdBlankSubstrate.getYear();
        if(StringUtils.isNotBlank(year)){
            fdBlankSubstrate.setYear(null);
        }

        QueryWrapper<FdBlankSubstrate> queryWrapper = QueryGenerator.initQueryWrapper(fdBlankSubstrate, req.getParameterMap());
        if (StringUtils.isNotBlank(trackingCode)) {
            queryWrapper.like("tracking_code", trackingCode);
        }
        if (StringUtils.isNotBlank(contractCode)) {
            queryWrapper.like("contract_code", contractCode);
        }
        if(StringUtils.isNotBlank(year)&&!year.equals("null")){
            queryWrapper.eq("YEAR(document_date)", year);
        }
        queryWrapper.lambda().eq(FdBlankSubstrate::getDelFlag, "0");
        queryWrapper.orderByDesc("document_date");
        Page<FdBlankSubstrate> page = new Page<FdBlankSubstrate>(pageNo, pageSize);
        IPage<FdBlankSubstrate> pageList = fdBlankSubstrateService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdBlankSubstrate
     * @return
     */
    @PostMapping(value = "/add")
    @NotDuplicate
    public Result<FdBlankSubstrate> add(@RequestBody @Valid FdBlankSubstrate fdBlankSubstrate, BindingResult bindingResult) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdBlankSubstrateService.save(fdBlankSubstrate);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败" + e.getMessage());
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdBlankSubstrate
     * @return
     */
    @PutMapping(value = "/edit")
    @NotDuplicate
    public Result<FdBlankSubstrate> edit(@RequestBody @Valid FdBlankSubstrate fdBlankSubstrate, BindingResult bindingResult) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        FdBlankSubstrate fdBlankSubstrateEntity = fdBlankSubstrateService.getById(fdBlankSubstrate.getId());
        if (fdBlankSubstrateEntity == null) {
            result.error500("未找到对应实体");
        } else {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            boolean ok = fdBlankSubstrateService.updateById(fdBlankSubstrate);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    // @DeleteMapping(value = "/delete")
    // public Result<FdBlankSubstrate> delete(@RequestParam(name = "id", required = true) String id) {
    //     Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
    //     FdBlankSubstrate fdBlankSubstrate = fdBlankSubstrateService.getById(id);
    //     if (fdBlankSubstrate == null) {
    //         result.error500("未找到对应实体");
    //     } else {
    //         boolean ok = fdBlankSubstrateService.removeById(id);
    //         if (ok) {
    //             result.success("删除成功!");
    //         }
    //     }
    //
    //     return result;
    // }

    /***
     * @Title:删除
            * @Description:TODO
            * @param id
            * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrate>
            * @exception
            * @author 陈创
            * @date 2019/11/1 17:13
            * @updatehistory:
     *   2019/11/1 17:13 删除
            */
    @DeleteMapping(value = "/delete")
    @NotDuplicate
    @RequiresPermissions("blank-substrate:delete")
    public Result<FdBlankSubstrate> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        FdBlankSubstrate fdBlankSubstrate = fdBlankSubstrateService.getById(id);
        if (fdBlankSubstrate == null) {
            result.error500("未找到对应实体");
        } else {
            fdBlankSubstrate.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
            boolean ok = fdBlankSubstrateService.updateById(fdBlankSubstrate);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdBlankSubstrate> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdBlankSubstrateService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdBlankSubstrate> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        FdBlankSubstrate fdBlankSubstrate = fdBlankSubstrateService.getById(id);
        if (fdBlankSubstrate == null) {
            result.error500("未找到对应实体");
        } else {
            if (fdBlankSubstrate.getSubstrateType()!=null && !fdBlankSubstrate.getSubstrateType().equals("")){
                //基质类型
                List<String> list = Arrays.asList(fdBlankSubstrate.getSubstrateType().split(","));
                //根据基质类型ID获取相关信息
                List<FdMatrixType> fdMatrixTypes = fdMatrixTypeMapper.selectBatchIds(list);
                //基质类型名称
                String SubstrateTypeName = "";
                //循环获取
                for(FdMatrixType fdMatrixType:fdMatrixTypes){
                    if (SubstrateTypeName == ""){
                        SubstrateTypeName = fdMatrixType.getName();
                    }else {
                        SubstrateTypeName += ","+fdMatrixType.getName();
                    }
                }
                fdBlankSubstrate.setSubstrateTypeName(SubstrateTypeName);
            }
            if (oConvertUtils.isNotEmpty(fdBlankSubstrate.getPkSubstrateCollectionHospital())) {
                //截取医院id
                List<String> list1 = Arrays.asList(fdBlankSubstrate.getPkSubstrateCollectionHospital().split(","));
                //根据医院id获取相关信息
                List<FdHospital> hospitalList = fdHospitalMapper.selectBatchIds(list1);
                //医院名称
                String hospitalName = "";
                //循环获取名称
                for (FdHospital fdHospital : hospitalList) {
                    if (hospitalName == "") {
                        hospitalName = fdHospital.getName();
                    } else {
                        hospitalName += "," + fdHospital.getName();
                    }
                }
                fdBlankSubstrate.setSubstrateCollectionHospital(hospitalName);
            }
            //空白基质领用日期 文件上传
            LambdaQueryWrapper<FdBlankSubstrateRecipientRecord> queryWrapper = new LambdaQueryWrapper<FdBlankSubstrateRecipientRecord>();
            queryWrapper.eq(FdBlankSubstrateRecipientRecord::getPkBlankSubstrate,fdBlankSubstrate.getId());
            queryWrapper.eq(FdBlankSubstrateRecipientRecord::getDelFlag,"0");
            queryWrapper.orderByAsc(FdBlankSubstrateRecipientRecord::getSort);
            List<FdBlankSubstrateRecipientRecord> fdBlankSubstrateRecipientRecordList = fdBlankSubstrateRecipientRecordService.list(queryWrapper);
            fdBlankSubstrate.setFdBlankSubstrateRecipientRecordList(fdBlankSubstrateRecipientRecordList);
            Map<String,Object> msp = Maps.newHashMap();
            msp.put("del_flag","0");
            msp.put("pk_blank_substrate",fdBlankSubstrate.getId());
            List<FdBlankSubstrateFileUpload> fdBlankSubstrateFileUploads = (List<FdBlankSubstrateFileUpload>)fdBlankSubstrateFileUploadService.listByMap(msp);
            fdBlankSubstrate.setFdBlankSubstrateFileUploadList(fdBlankSubstrateFileUploads);
            result.setResult(fdBlankSubstrate);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdBlankSubstrate> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdBlankSubstrate fdBlankSubstrate = JSON.parseObject(deString, FdBlankSubstrate.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdBlankSubstrate, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdBlankSubstrate> pageList = fdBlankSubstrateService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "空白基质管理列表");
        mv.addObject(NormalExcelConstants.CLASS, FdBlankSubstrate.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("空白基质管理列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdBlankSubstrate> listFdBlankSubstrates = ExcelImportUtil.importExcel(file.getInputStream(), FdBlankSubstrate.class, params);
                for (FdBlankSubstrate fdBlankSubstrateExcel : listFdBlankSubstrates) {
                    fdBlankSubstrateService.save(fdBlankSubstrateExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdBlankSubstrates.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrate>
     * @Author gurui
     * @Description //TODO 新增空白基质管理表 同时添加  空白基质文件上传 空白基质领用记录
     * @Date 11:32 2019/6/11
     * @Param [fdBlankSubstrate, bindingResult]
     **/
    @PostMapping(value = "/addAll")
    @NotDuplicate
    @RequiresPermissions(value = {"blank-substrate:add","blank-substrate:update"},logical = Logical.OR)
    public Result<FdBlankSubstrate> addAll(@RequestBody @Valid FdBlankSubstrate fdBlankSubstrate, BindingResult bindingResult) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdBlankSubstrateService.addAll(fdBlankSubstrate);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败" + e.getMessage());
        }
        return result;
    }


    /**
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrate>
     * @Author gurui
     * @Description //TODO 根据合同id查询相关数据
     * @Date 10:47 2019/6/26
     * @Param [contractId]
     **/
    @GetMapping(value = "/queryBlankSubstrateByConId")
    public Result<FdBlankSubstrate> queryBlankSubstrateByConId(@RequestParam(name = "pkProjectFile", required = true) String pkProjectFile) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        FdBlankSubstrate fdBlankSubstrate = fdBlankSubstrateService.queryBlankSubstrateByConId(pkProjectFile);
        if (fdBlankSubstrate == null) {
            result.error500("未找到对应实体");
        } else {

            result.setResult(fdBlankSubstrate);
            result.setSuccess(true);
        }
        return result;
    }


    /**
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdBlankSubstrate>
     * @Author gurui
     * @Description //TODO 删除空白基质 同时删除 领用记录 文件上传
     * @Date 14:28 2019/6/27
     * @Param [id]
     **/
    @DeleteMapping(value = "/deleteAll")
    public Result<FdBlankSubstrate> deleteAll(@RequestParam(name = "id", required = true) String id) {
        Result<FdBlankSubstrate> result = new Result<FdBlankSubstrate>();
        FdBlankSubstrate fdBlankSubstrate = fdBlankSubstrateService.getById(id);
        if (fdBlankSubstrate == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdBlankSubstrateService.deleteAll(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * @Author gurui
     * @Description //TODO  查询已审核项目档案的合同号
     * @Date 10:38 2019/7/12
     * @Param []
     * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.projectManagement.entity.FdContract>>
     **/
    @GetMapping(value = "/getFileContractList")
    public Result<List<FdContract>> getFileContractList(){
        Result<List<FdContract>> result=new Result<>();
        result.setResult(fdBlankSubstrateService.getFileContractList());
        result.setSuccess(true);
        return result;
    }




}
