package org.jeecg.modules.sampleAndConsumableManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.Logical;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.constant.CommonConstant;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdPersonnel;
import org.jeecg.modules.basedata.service.IFdPersonnelService;
import org.jeecg.modules.projectManagement.entity.FdContract;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdColumnManagement;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdColumnManagementRecipientRecord;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdColumnManagementRecipientRecordService;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdColumnManagementService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 色谱柱管理
 * @author： jeecg-boot
 * @date： 2019-06-11
 * @version： V1.0
 */
@RestController
@RequestMapping("/sampleAndConsumableManagement/fdColumnManagement")
@Slf4j
public class FdColumnManagementController {
    @Autowired
    private IFdColumnManagementService fdColumnManagementService;

    @Resource
    private IFdPersonnelService fdPersonnelService;

    @Resource
    private IFdColumnManagementRecipientRecordService fdColumnManagementRecipientRecordService;

    /**
     * 分页列表查询
     *
     * @param fdColumnManagement
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    @RequiresPermissions("url:column-management:view")
    public Result<IPage<FdColumnManagement>> queryPageList(FdColumnManagement fdColumnManagement,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<FdColumnManagement>> result = new Result<IPage<FdColumnManagement>>();
        String columnName = fdColumnManagement.getColumnName();
        if (StringUtils.isNotBlank(fdColumnManagement.getColumnName())){
            fdColumnManagement.setColumnName(null);
        }
        String lotNo = fdColumnManagement.getLotNo();
        if (StringUtils.isNotBlank(fdColumnManagement.getLotNo())){
            fdColumnManagement.setLotNo(null);
        }
        QueryWrapper<FdColumnManagement> queryWrapper = QueryGenerator.initQueryWrapper(fdColumnManagement, req.getParameterMap());
        if (StringUtils.isNotBlank(columnName)) {
            queryWrapper.like("column_name", columnName);
        }
        if (StringUtils.isNotBlank(lotNo)) {
            queryWrapper.like("lot_no", lotNo);
        }
        // queryWrapper.lambda().eq(FdColumnManagement::getDelFlag,"0");
        queryWrapper.lambda().eq(FdColumnManagement::getDelFlag,"0");
        queryWrapper.orderByDesc("create_time");
        Page<FdColumnManagement> page = new Page<FdColumnManagement>(pageNo, pageSize);
        IPage<FdColumnManagement> pageList = fdColumnManagementService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdColumnManagement
     * @return
     */
    @PostMapping(value = "/add")
    @NotDuplicate
    public Result<FdColumnManagement> add(@RequestBody @Valid FdColumnManagement fdColumnManagement, BindingResult bindingResult) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdColumnManagementService.save(fdColumnManagement);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdColumnManagement
     * @return
     */
    @PutMapping(value = "/edit")
    @NotDuplicate
    public Result<FdColumnManagement> edit(@RequestBody @Valid FdColumnManagement fdColumnManagement, BindingResult bindingResult) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        FdColumnManagement fdColumnManagementEntity = fdColumnManagementService.getById(fdColumnManagement.getId());
        if (fdColumnManagementEntity == null) {
            result.error500("未找到对应实体");
        } else {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            boolean ok = fdColumnManagementService.updateById(fdColumnManagement);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    // @DeleteMapping(value = "/delete")
    // public Result<FdColumnManagement> delete(@RequestParam(name = "id", required = true) String id) {
    //     Result<FdColumnManagement> result = new Result<FdColumnManagement>();
    //     FdColumnManagement fdColumnManagement = fdColumnManagementService.getById(id);
    //     if (fdColumnManagement == null) {
    //         result.error500("未找到对应实体");
    //     } else {
    //         boolean ok = fdColumnManagementService.removeById(id);
    //         if (ok) {
    //             result.success("删除成功!");
    //         }
    //     }
    //
    //     return result;
    // }

    /***
     * @Title:删除
            * @Description:TODO
            * @param id
            * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdColumnManagement>
            * @exception
            * @author 陈创
            * @date 2019/11/1 17:14
            * @updatehistory:
     *   2019/11/1 17:14 删除
            */
    @DeleteMapping(value = "/delete")
    @NotDuplicate
    @RequiresPermissions("column-management:delete")
    public Result<FdColumnManagement> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        FdColumnManagement fdColumnManagement = fdColumnManagementService.getById(id);
        if (fdColumnManagement == null) {
            result.error500("未找到对应实体");
        } else {
            fdColumnManagement.setDelFlag(CommonConstant.DEL_FLAG_1.toString());
            boolean ok = fdColumnManagementService.updateById(fdColumnManagement);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdColumnManagement> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdColumnManagementService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdColumnManagement> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        FdColumnManagement fdColumnManagement = fdColumnManagementService.getById(id);
        if (fdColumnManagement == null) {
            result.error500("未找到对应实体");
        } else {
//            Map<String,Object> msp = Maps.newHashMap();
//            msp.put("del_flag","0");
//            msp.put("pk_column_management",fdColumnManagement.getId());
//            List<FdColumnManagementRecipientRecord> fdColumnManagementRecipientRecords = (List<FdColumnManagementRecipientRecord>)fdColumnManagementRecipientRecordService.listByMap(msp);
//            fdColumnManagement.setFdColumnManagementRecipientRecordList(fdColumnManagementRecipientRecords);
            LambdaQueryWrapper<FdColumnManagementRecipientRecord> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(FdColumnManagementRecipientRecord::getPkColumnManagement,fdColumnManagement.getId());
            queryWrapper.eq(FdColumnManagementRecipientRecord::getDelFlag,"0");
            queryWrapper.orderByAsc(FdColumnManagementRecipientRecord::getSort);
            List<FdColumnManagementRecipientRecord> fdColumnManagementRecipientRecordList = fdColumnManagementRecipientRecordService.list(queryWrapper);
            fdColumnManagement.setFdColumnManagementRecipientRecordList(fdColumnManagementRecipientRecordList);
            //根据数据字典查询合同方
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkContractParty())){
                fdColumnManagement.setContractPartyName(getContractPartyName(fdColumnManagement.getPkContractParty()));
            }
            //根据数据字典查询粒径
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkParticleSize())){
                fdColumnManagement.setParticleSizeName(getParticleSizeName(fdColumnManagement.getPkParticleSize()));
            }
            //根据数据字典查询柱型
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkColumnType())){
                fdColumnManagement.setColumnTypeName(getColumnTypeName(fdColumnManagement.getPkColumnType()));
            }
            //根据数据字典查询柱长
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkColumnLength())){
                fdColumnManagement.setColumnLengthName(getColumnLengthName(fdColumnManagement.getPkColumnLength()));
            }
            //根据数据字典查询柱径
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkColumnDiameter())){
                fdColumnManagement.setColumnDiameterName(getColumnDiameter(fdColumnManagement.getPkColumnDiameter()));
            }
            result.setResult(fdColumnManagement);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdColumnManagement> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdColumnManagement fdColumnManagement = JSON.parseObject(deString, FdColumnManagement.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdColumnManagement, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdColumnManagement> pageList = fdColumnManagementService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "色谱柱管理列表");
        mv.addObject(NormalExcelConstants.CLASS, FdColumnManagement.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("色谱柱管理列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdColumnManagement> listFdColumnManagements = ExcelImportUtil.importExcel(file.getInputStream(), FdColumnManagement.class, params);
                for (FdColumnManagement fdColumnManagementExcel : listFdColumnManagements) {
                    fdColumnManagementService.save(fdColumnManagementExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdColumnManagements.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * @Author gurui
     * @Description //TODO 添加色谱柱管理 同时添加 色谱柱管理领用记录
     * @Date 9:18 2019/6/12
     * @Param [fdColumnManagement, bindingResult]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdColumnManagement>
     **/
    @PostMapping(value = "/addAll")
    @NotDuplicate
    @RequiresPermissions(value = {"column-management:add","column-management:update"},logical = Logical.OR)
    public Result<FdColumnManagement> addAll(@RequestBody @Valid FdColumnManagement fdColumnManagement, BindingResult bindingResult) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdColumnManagementService.addAll(fdColumnManagement);
            //根据数据字典查询合同方
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkContractParty())){
                fdColumnManagement.setContractPartyName(getContractPartyName(fdColumnManagement.getPkContractParty()));
            }
            //根据数据字典查询粒径
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkParticleSize())){
                fdColumnManagement.setParticleSizeName(getParticleSizeName(fdColumnManagement.getPkParticleSize()));
            }
            //根据数据字典查询柱型
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkColumnType())){
                fdColumnManagement.setColumnTypeName(getColumnTypeName(fdColumnManagement.getPkColumnType()));
            }
            //根据数据字典查询柱长
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkColumnLength())){
                fdColumnManagement.setColumnLengthName(getColumnLengthName(fdColumnManagement.getPkColumnLength()));
            }
            //根据数据字典查询柱径
            if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkColumnDiameter())){
                fdColumnManagement.setColumnDiameterName(getColumnDiameter(fdColumnManagement.getPkColumnDiameter()));
            }
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * @Author gurui
     * @Description //TODO 根据合同号查询相关数据
     * @Date 11:03 2019/6/27
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdColumnManagement>
     **/
    @GetMapping(value = "/queryColumnByConId")
    public Result<FdColumnManagement> queryColumnByConId(@RequestParam(name = "pkProjectFile", required = true)String pkProjectFile) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        FdColumnManagement fdColumnManagement = fdColumnManagementService.queryColumnByConId(pkProjectFile);
        if (oConvertUtils.isNotEmpty(fdColumnManagement.getPkContractParty())){
            fdColumnManagement.setContractPartyName(getContractPartyName(fdColumnManagement.getPkContractParty()));
        }
        if (fdColumnManagement == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdColumnManagement);
            result.setSuccess(true);
        }
        return result;
    }



    /**
     * @Author gurui
     * @Description //TODO 删除色谱柱管理 同时删除色谱柱管理领用记录
     * @Date 13:23 2019/6/27
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdColumnManagement>
     **/
    @DeleteMapping(value = "/deleteAll")
    public Result<FdColumnManagement> deleteAll(@RequestParam(name = "id", required = true)String id) {
        Result<FdColumnManagement> result = new Result<FdColumnManagement>();
        FdColumnManagement fdColumnManagement = fdColumnManagementService.getById(id);
        if (fdColumnManagement == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdColumnManagementService.deleteAll(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }


    /**
     * @Author gurui
     * @Description //TODO  查询已审核项目档案的合同号
     * @Date 10:38 2019/7/12
     * @Param []
     * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.projectManagement.entity.FdContract>>
     **/
    @GetMapping(value = "/getFileContractList")
    public Result<List<FdContract>> getFileContractList(){
        Result<List<FdContract>> result=new Result<>();
        result.setResult(fdColumnManagementService.getFileContractList());
        result.setSuccess(true);
        return result;
    }


    //根据合同方id查询合同方名称
    public String getContractPartyName(@RequestParam(name = "pkContractParty", required = true)String pkContractParty){
        Result<String> result=new Result<>();
        String contractPartyName = fdColumnManagementService.getContractPartyName(pkContractParty);
        return contractPartyName;
    }

    //根据柱型id获取柱型名称(数据字典)
    public String getColumnTypeName(String pkColumnType){
        Result<String> result=new Result<>();
        String ColumnTypeName = fdColumnManagementService.getColumnTypeName(pkColumnType);
        return ColumnTypeName;
    }

    //根据粒径id获取粒径名称(数据字典)
    public String getParticleSizeName(String pkParticleSize){
        Result<String> result=new Result<>();
        String ParticleSizeName = fdColumnManagementService.getParticleSizeName(pkParticleSize);
        return ParticleSizeName;
    }

    //根据柱长id获取柱长名称(数据字典)
    public String getColumnLengthName(String pkColumnLength){
        Result<String> result=new Result<>();
        String ColumnLengthName = fdColumnManagementService.getColumnLengthName(pkColumnLength);
        return ColumnLengthName;
    }

    //根据柱径id获取柱径名称(数据字典)
    public String getColumnDiameter(String pkColumnDiameter){
        Result<String> result=new Result<>();
        String ColumnDiameter = fdColumnManagementService.getColumnDiameter(pkColumnDiameter);
        return ColumnDiameter;
    }


}
