package org.jeecg.modules.sampleAndConsumableManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.projectManagement.entity.FdContract;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdReagentManagement;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdReagentManagementRecipientRecord;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdReagentManagementRecipientRecordService;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdReagentManagementService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecg.modules.sampleAndConsumableManagement.service.IFdStandardProductsRecipientRecordService;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 试剂管理
 * @author： jeecg-boot
 * @date： 2019-06-12
 * @version： V1.0
 */
@RestController
@RequestMapping("/sampleAndConsumableManagement/fdReagentManagement")
@Slf4j
public class FdReagentManagementController {
    @Autowired
    private IFdReagentManagementService fdReagentManagementService;

    @Resource
    private IFdReagentManagementRecipientRecordService fdReagentManagementRecipientRecordService;

    /**
     * 分页列表查询
     *
     * @param fdReagentManagement
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    @RequiresPermissions("url:reagent-management:view")
    public Result<IPage<FdReagentManagement>> queryPageList(FdReagentManagement fdReagentManagement,
                                                            @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                            @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                            HttpServletRequest req) {
        Result<IPage<FdReagentManagement>> result = new Result<IPage<FdReagentManagement>>();
        if (StringUtils.isNotBlank(fdReagentManagement.getProjectName())) {
            fdReagentManagement.setProjectName("*" + fdReagentManagement.getProjectName() + "*");
        }
        QueryWrapper<FdReagentManagement> queryWrapper = QueryGenerator.initQueryWrapper(fdReagentManagement, req.getParameterMap());
        queryWrapper.lambda().eq(FdReagentManagement::getDelFlag, "0");
        queryWrapper.orderByDesc("create_time");
        Page<FdReagentManagement> page = new Page<FdReagentManagement>(pageNo, pageSize);
        IPage<FdReagentManagement> pageList = fdReagentManagementService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdReagentManagement
     * @return
     */
    @PostMapping(value = "/add")
    @NotDuplicate
    public Result<FdReagentManagement> add(@RequestBody @Valid FdReagentManagement fdReagentManagement, BindingResult bindingResult) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdReagentManagementService.save(fdReagentManagement);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败"+e.getMessage());
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdReagentManagement
     * @return
     */
    @PutMapping(value = "/edit")
    @NotDuplicate
    public Result<FdReagentManagement> edit(@RequestBody @Valid FdReagentManagement fdReagentManagement, BindingResult bindingResult) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        FdReagentManagement fdReagentManagementEntity = fdReagentManagementService.getById(fdReagentManagement.getId());
        if (fdReagentManagementEntity == null) {
            result.error500("未找到对应实体");
        } else {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            boolean ok = fdReagentManagementService.updateById(fdReagentManagement);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<FdReagentManagement> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        FdReagentManagement fdReagentManagement = fdReagentManagementService.getById(id);
        if (fdReagentManagement == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdReagentManagementService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdReagentManagement> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdReagentManagementService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdReagentManagement> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        FdReagentManagement fdReagentManagement = fdReagentManagementService.getById(id);
        if (fdReagentManagement == null) {
            result.error500("未找到对应实体");
        } else {
            Map<String,Object> msp = Maps.newHashMap();
            msp.put("del_flag","0");
            msp.put("pk_reagent_management",fdReagentManagement.getId());
            List<FdReagentManagementRecipientRecord> fdReagentManagementRecipientRecords = (List<FdReagentManagementRecipientRecord>)fdReagentManagementRecipientRecordService.listByMap(msp);
            fdReagentManagement.setFdReagentManagementRecipientRecordList(fdReagentManagementRecipientRecords);
            result.setResult(fdReagentManagement);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdReagentManagement> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdReagentManagement fdReagentManagement = JSON.parseObject(deString, FdReagentManagement.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdReagentManagement, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdReagentManagement> pageList = fdReagentManagementService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "试剂管理列表");
        mv.addObject(NormalExcelConstants.CLASS, FdReagentManagement.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("试剂管理列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdReagentManagement> listFdReagentManagements = ExcelImportUtil.importExcel(file.getInputStream(), FdReagentManagement.class, params);
                for (FdReagentManagement fdReagentManagementExcel : listFdReagentManagements) {
                    fdReagentManagementService.save(fdReagentManagementExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdReagentManagements.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * @Author gurui
     * @Description //TODO 添加 试剂管理同时添加  试剂管理领用记录
     * @Date 10:49 2019/6/12
     * @Param [fdReagentManagement, bindingResult]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdReagentManagement>
     **/
    @PostMapping(value = "/addAll")
    @NotDuplicate
    public Result<FdReagentManagement> addAll(@RequestBody @Valid FdReagentManagement fdReagentManagement,BindingResult bindingResult) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdReagentManagementService.addAll(fdReagentManagement);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败"+e.getMessage());
        }
        return result;
    }


    /**
     * @Author gurui
     * @Description //TODO 根据合同id查询相关数据
     * @Date 13:41 2019/6/27
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdReagentManagement>
     **/
    @GetMapping(value = "/queryReagentByConId")
    public Result<FdReagentManagement> queryReagentByConId(@RequestParam(name = "contractId", required = true)String contractId) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        FdReagentManagement fdReagentManagement = fdReagentManagementService.queryReagentByConId(contractId);
        if (fdReagentManagement == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdReagentManagement);
            result.setSuccess(true);
        }
        return result;
    }


    /**
     * @Author gurui
     * @Description //TODO 删除试剂管理同时删除领用记录
     * @Date 15:22 2019/6/27
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdReagentManagement>
     **/
    @DeleteMapping(value = "/deleteAll")
    public Result<FdReagentManagement> deleteAll(@RequestParam(name = "id", required = true) String id) {
        Result<FdReagentManagement> result = new Result<FdReagentManagement>();
        FdReagentManagement fdReagentManagement = fdReagentManagementService.getById(id);
        if (fdReagentManagement == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdReagentManagementService.deleteAll(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * @Author gurui
     * @Description //TODO  查询已审核项目档案的合同号
     * @Date 10:38 2019/7/12
     * @Param []
     * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.projectManagement.entity.FdContract>>
     **/
    @GetMapping(value = "/getFileContractList")
    public Result<List<FdContract>> getFileContractList(){
        Result<List<FdContract>> result=new Result<>();
        result.setResult(fdReagentManagementService.getFileContractList());
        result.setSuccess(true);
        return result;
    }


}
