package org.jeecg.modules.sampleAndConsumableManagement.controller;

import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;

import com.google.common.collect.Maps;
import org.apache.commons.lang.StringUtils;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.aspect.annotation.NotDuplicate;
import org.jeecg.common.system.query.QueryGenerator;
import org.jeecg.common.util.oConvertUtils;
import org.jeecg.modules.basedata.entity.FdMatrixType;
import org.jeecg.modules.basedata.mapper.FdMatrixTypeMapper;
import org.jeecg.modules.projectManagement.entity.FdContract;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdStandardProducts;
import org.jeecg.modules.sampleAndConsumableManagement.entity.FdStandardProductsRecipientRecord;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdStandardProductsRecipientRecordService;
import org.jeecg.modules.sampleAndConsumableManagement.service.IFdStandardProductsService;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.extern.slf4j.Slf4j;

import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;
import com.alibaba.fastjson.JSON;

/**
 * @Title: Controller
 * @Description: 标准品管理
 * @author： jeecg-boot
 * @date： 2019-06-12
 * @version： V1.0
 */
@RestController
@RequestMapping("/sampleAndConsumableManagement/fdStandardProducts")
@Slf4j
public class FdStandardProductsController {
    @Autowired
    private IFdStandardProductsService fdStandardProductsService;

    @Resource
    private FdMatrixTypeMapper fdMatrixTypeMapper;

    @Resource
    private IFdStandardProductsRecipientRecordService fdStandardProductsRecipientRecordService;

    /**
     * 分页列表查询
     *
     * @param fdStandardProducts
     * @param pageNo
     * @param pageSize
     * @param req
     * @return
     */
    @GetMapping(value = "/list")
    public Result<IPage<FdStandardProducts>> queryPageList(FdStandardProducts fdStandardProducts,
                                                           @RequestParam(name = "pageNo", defaultValue = "1") Integer pageNo,
                                                           @RequestParam(name = "pageSize", defaultValue = "10") Integer pageSize,
                                                           HttpServletRequest req) {
        Result<IPage<FdStandardProducts>> result = new Result<IPage<FdStandardProducts>>();
        if (StringUtils.isNotBlank(fdStandardProducts.getProjectName())){
            fdStandardProducts.setProjectName("*"+fdStandardProducts.getProjectName()+"*");
        }
        QueryWrapper<FdStandardProducts> queryWrapper = QueryGenerator.initQueryWrapper(fdStandardProducts, req.getParameterMap());
        queryWrapper.lambda().eq(FdStandardProducts::getDelFlag,"0");
        queryWrapper.orderByDesc("create_time");
        Page<FdStandardProducts> page = new Page<FdStandardProducts>(pageNo, pageSize);
        IPage<FdStandardProducts> pageList = fdStandardProductsService.page(page, queryWrapper);
        result.setSuccess(true);
        result.setResult(pageList);
        return result;
    }

    /**
     * 添加
     *
     * @param fdStandardProducts
     * @return
     */
    @PostMapping(value = "/add")
    @NotDuplicate
    public Result<FdStandardProducts> add(@RequestBody @Valid FdStandardProducts fdStandardProducts, BindingResult bindingResult) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdStandardProductsService.save(fdStandardProducts);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }

    /**
     * 编辑
     *
     * @param fdStandardProducts
     * @return
     */
    @PutMapping(value = "/edit")
    @NotDuplicate
    public Result<FdStandardProducts> edit(@RequestBody @Valid FdStandardProducts fdStandardProducts, BindingResult bindingResult) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        FdStandardProducts fdStandardProductsEntity = fdStandardProductsService.getById(fdStandardProducts.getId());
        if (fdStandardProductsEntity == null) {
            result.error500("未找到对应实体");
        } else {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            boolean ok = fdStandardProductsService.updateById(fdStandardProducts);
            //TODO 返回false说明什么？
            if (ok) {
                result.success("修改成功!");
            }
        }

        return result;
    }

    /**
     * 通过id删除
     *
     * @param id
     * @return
     */
    @DeleteMapping(value = "/delete")
    public Result<FdStandardProducts> delete(@RequestParam(name = "id", required = true) String id) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        FdStandardProducts fdStandardProducts = fdStandardProductsService.getById(id);
        if (fdStandardProducts == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdStandardProductsService.removeById(id);
            if (ok) {
                result.success("删除成功!");
            }
        }

        return result;
    }

    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @DeleteMapping(value = "/deleteBatch")
    public Result<FdStandardProducts> deleteBatch(@RequestParam(name = "ids", required = true) String ids) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        if (ids == null || "".equals(ids.trim())) {
            result.error500("参数不识别！");
        } else {
            this.fdStandardProductsService.removeByIds(Arrays.asList(ids.split(",")));
            result.success("删除成功!");
        }
        return result;
    }

    /**
     * 通过id查询
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/queryById")
    public Result<FdStandardProducts> queryById(@RequestParam(name = "id", required = true) String id) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        FdStandardProducts fdStandardProducts = fdStandardProductsService.getById(id);
        if (fdStandardProducts == null) {
            result.error500("未找到对应实体");
        } else {
            //基质分类名称
            if (fdStandardProducts.getSubstrateType()!=null){
                List<String> list = Arrays.asList(fdStandardProducts.getSubstrateType().split(","));
                List<FdMatrixType> matrixTypes = fdMatrixTypeMapper.selectBatchIds(list);
                String matrixType = "";
                for (FdMatrixType fdMatrixType:matrixTypes){
                    if (matrixType==""){
                        matrixType = fdMatrixType.getName();
                    }else{
                        matrixType += ","+fdMatrixType.getName();
                    }
                }
                fdStandardProducts.setSubstrateTypeName(matrixType);
            }
            //查询标准品领用
            Map<String,Object> msp = Maps.newHashMap();
            msp.put("del_flag","0");
            msp.put("pk_standard_products",fdStandardProducts.getId());
            List<FdStandardProductsRecipientRecord> fdStandardProductsRecipientRecords = (List<FdStandardProductsRecipientRecord>)fdStandardProductsRecipientRecordService.listByMap(msp);
            fdStandardProducts.setFdStandardProductsRecipientRecordList(fdStandardProductsRecipientRecords);
            result.setResult(fdStandardProducts);
            result.setSuccess(true);
        }
        return result;
    }

    /**
     * 导出excel
     *
     * @param request
     * @param response
     */
    @RequestMapping(value = "/exportXls")
    public ModelAndView exportXls(HttpServletRequest request, HttpServletResponse response) {
        // Step.1 组装查询条件
        QueryWrapper<FdStandardProducts> queryWrapper = null;
        try {
            String paramsStr = request.getParameter("paramsStr");
            if (oConvertUtils.isNotEmpty(paramsStr)) {
                String deString = URLDecoder.decode(paramsStr, "UTF-8");
                FdStandardProducts fdStandardProducts = JSON.parseObject(deString, FdStandardProducts.class);
                queryWrapper = QueryGenerator.initQueryWrapper(fdStandardProducts, request.getParameterMap());
            }
        } catch (UnsupportedEncodingException e) {
            e.printStackTrace();
        }

        //Step.2 AutoPoi 导出Excel
        ModelAndView mv = new ModelAndView(new JeecgEntityExcelView());
        List<FdStandardProducts> pageList = fdStandardProductsService.list(queryWrapper);
        //导出文件名称
        mv.addObject(NormalExcelConstants.FILE_NAME, "标准品管理列表");
        mv.addObject(NormalExcelConstants.CLASS, FdStandardProducts.class);
        mv.addObject(NormalExcelConstants.PARAMS, new ExportParams("标准品管理列表数据", "导出人:Jeecg", "导出信息"));
        mv.addObject(NormalExcelConstants.DATA_LIST, pageList);
        return mv;
    }

    /**
     * 通过excel导入数据
     *
     * @param request
     * @param response
     * @return
     */
    @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
    public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
        for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
            MultipartFile file = entity.getValue();// 获取上传文件对象
            ImportParams params = new ImportParams();
            params.setTitleRows(2);
            params.setHeadRows(1);
            params.setNeedSave(true);
            try {
                List<FdStandardProducts> listFdStandardProductss = ExcelImportUtil.importExcel(file.getInputStream(), FdStandardProducts.class, params);
                for (FdStandardProducts fdStandardProductsExcel : listFdStandardProductss) {
                    fdStandardProductsService.save(fdStandardProductsExcel);
                }
                return Result.ok("文件导入成功！数据行数：" + listFdStandardProductss.size());
            } catch (Exception e) {
                log.error(e.getMessage());
                return Result.error("文件导入失败！");
            } finally {
                try {
                    file.getInputStream().close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        return Result.ok("文件导入失败！");
    }

    /**
     * @Author gurui
     * @Description //TODO 添加标准品管理 同时添加  标准品管理领用记录
     * @Date 14:03 2019/6/12
     * @Param [fdStandardProducts, bindingResult]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdStandardProducts>
     **/
    @PostMapping(value = "/addAll")
    @NotDuplicate
    public Result<FdStandardProducts> addAll(@RequestBody @Valid FdStandardProducts fdStandardProducts, BindingResult bindingResult) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        try {
            StringBuilder sb = new StringBuilder();
            if (bindingResult.hasErrors()) {
                //记录错误信息
                bindingResult.getAllErrors().stream().forEach(error -> sb.append(error.getDefaultMessage() + "<br/>"));
                result.error500(sb.toString());
                return result;
            }
            fdStandardProductsService.addAll(fdStandardProducts);
            result.success("添加成功！");
        } catch (Exception e) {
            e.printStackTrace();
            log.info(e.getMessage());
            result.error500("操作失败");
        }
        return result;
    }


    /**
     * @Author gurui
     * @Description //TODO 根据合同id查询相关信息
     * @Date 15:46 2019/6/26
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdStandardProducts>
     **/
    @GetMapping(value = "/queryStandardByConId")
    public Result<FdStandardProducts> queryStandardByConId(String contractId) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        FdStandardProducts fdStandardProducts = fdStandardProductsService.queryStandardByConId(contractId);
        if (fdStandardProducts == null) {
            result.error500("未找到对应实体");
        } else {
            result.setResult(fdStandardProducts);
            result.setSuccess(true);
        }
        return result;
    }



    /**
     * @Author gurui
     * @Description //TODO 删除标准品管理同时删除领用记录
     * @Date 9:27 2019/6/27
     * @Param [id]
     * @return org.jeecg.common.api.vo.Result<org.jeecg.modules.sampleAndConsumableManagement.entity.FdStandardProducts>
     **/
    @DeleteMapping(value = "/deleteAll")
    public Result<FdStandardProducts> deleteAll(@RequestParam(name = "id", required = true) String id) {
        Result<FdStandardProducts> result = new Result<FdStandardProducts>();
        FdStandardProducts fdStandardProducts = fdStandardProductsService.getById(id);
        if (fdStandardProducts == null) {
            result.error500("未找到对应实体");
        } else {
            boolean ok = fdStandardProductsService.deleteAll(id);
            if (ok) {
                result.success("删除成功!");
            }
        }
        return result;
    }

    /**
     * @Author gurui
     * @Description //TODO  查询已审核项目档案的合同号
     * @Date 10:38 2019/7/12
     * @Param []
     * @return org.jeecg.common.api.vo.Result<java.util.List<org.jeecg.modules.projectManagement.entity.FdContract>>
     **/
    @GetMapping(value = "/getFileContractList")
    public Result<List<FdContract>> getFileContractList(){
        Result<List<FdContract>> result=new Result<>();
        result.setResult(fdStandardProductsService.getFileContractList());
        result.setSuccess(true);
        return result;
    }



}
